@@ -32,7 +32,9 @@
 #include "chuck_compile.h"
 #include "chuck_lang.h"
 #include "chuck_errmsg.h"
+#include "chuck.h"
 #include "chuck_io.h"
+
 #ifndef  __DISABLE_OTF_SERVER__
 #include "chuck_otf.h"
 #endif
@@ -49,23 +51,21 @@
 #include "ulib_machine.h"
 #include "ulib_math.h"
 #include "ulib_std.h"
+#include "util_string.h"
 
 #ifndef __DISABLE_NETWORK__
 #include "ulib_opsc.h"
 #endif
 
-#if defined(__PLATFORM_WIN32__)
+#if defined(__PLATFORM_WINDOWS__)
   #include "dirent_win32.h"
-#else
-  #ifndef __DISABLE_WORDEXP__
-  #include <wordexp.h> // 1.5.0.0 (ge) added
-  #endif
 #endif
 
-//#if defined(__WINDOWS_PTHREAD__)
 #include <sys/stat.h>
-//#endif
 
+#include <string>
+#include <vector>
+#include <algorithm>
 using namespace std;
 
 
@@ -94,6 +94,8 @@ Chuck_Compiler::Chuck_Compiler()
 
     // REFACTOR-2017: add carrier
     m_carrier = NULL;
+    // initialize
+    m_auto_depend = FALSE;
 
     // origin hint | 1.5.0.0 (ge) added
     m_originHint = te_originUnknown;
@@ -121,18 +123,22 @@ Chuck_Compiler::~Chuck_Compiler()
 //-----------------------------------------------------------------------------
 t_CKBOOL Chuck_Compiler::initialize()
 {
+    // this should have been set by initialization
+    assert( m_carrier != NULL );
+
     // set origin hint
     m_originHint = te_originBuiltin;
 
     // allocate the type checker
-    Chuck_Env * env = type_engine_init( m_carrier );
-    // add reference
-    env->add_ref();
+    if( !type_engine_init( m_carrier ) )
+        goto error;
+    // ensure carrier's env is set in type_engine_init
+    assert( m_carrier->env != NULL );
 
     // allocate the emitter
-    emitter = emit_engine_init( env );
+    emitter = emit_engine_init( m_carrier->env );
     // add reference
-    emitter->add_ref();
+    CK_SAFE_ADD_REF( emitter );
     // set auto depend to 0
     m_auto_depend = FALSE;
 
@@ -169,21 +175,35 @@ void Chuck_Compiler::shutdown()
     // push indent
     EM_pushlog();
 
-    // TODO: free
-    type_engine_shutdown( env() );
-    // TODO: check if emitter gets cleaned up
-    // emit_engine_shutdown( emitter );
-    emitter = NULL;
+    // need an actual carrier
+    if( m_carrier != NULL )
+    {
+        // free type engine env
+        type_engine_shutdown( m_carrier );
+        // check the pointer is now NULL
+        assert( m_carrier->env == NULL );
+        // zero out the carrier reference | 1.5.1.1
+        m_carrier = NULL;
+    }
+
+    // free emitter (and set emitter to NULL)
+    emit_engine_shutdown( emitter );
+    // check the pointer is now NULL
+    assert( emitter == NULL );
+
+    // clear more state
     code = NULL;
     m_auto_depend = FALSE;
     m_recent.clear();
 
-    for(std::list<Chuck_DLL *>::iterator i = m_dlls.begin();
-        i != m_dlls.end(); i++)
+    // clean up DLLs
+    std::list<Chuck_DLL *>::iterator iter;
+    for( iter = m_dlls.begin(); iter != m_dlls.end(); iter++ )
     {
-        delete (*i);
+        // delete each DLL
+        CK_SAFE_DELETE( *iter );
     }
-
+    // clear the list
     m_dlls.clear();
 
     // pop indent
@@ -267,28 +287,35 @@ void Chuck_Compiler::set_auto_depend( t_CKBOOL v )
 // name: go()
 // desc: parse, type-check, and emit a program
 //-----------------------------------------------------------------------------
-t_CKBOOL Chuck_Compiler::go( const string & filename, FILE * fd, const char * str_src, const string & full_path )
+t_CKBOOL Chuck_Compiler::go( const string & filename,
+                             const string & full_path,
+                             const string & codeLiteral )
 {
-    t_CKBOOL ret = TRUE;
-    // Chuck_Context * context = NULL;
+    // hold return value
+    t_CKBOOL ret = FALSE;
 
+    // clear any error messages
     EM_reset_msg();
+    // set the chuck | 1.5.0.5 (ge) added
+    EM_set_current_chuck( this->carrier()->chuck );
 
     // check to see if resolve dependencies automatically
     if( !m_auto_depend )
     {
         // normal (note: full_path added 1.3.0.0)
-        ret = this->do_normal_depend( filename, fd, str_src, full_path );
+        ret = this->do_normal_depend( filename, codeLiteral, full_path );
     }
     else // auto depend
     {
         // call auto-depend compile (1.4.1.0)
-        ret = this->do_auto_depend( filename, fd, str_src, full_path );
+        ret = this->do_auto_depend( filename, codeLiteral, full_path );
     }
 
     // 1.4.1.0 (ge) | added to unset the fileName reference, which determines
     // how messages print to console (e.g., [file.ck]: or [chuck]:)
-    EM_change_file( NULL );
+    // 1.5.0.5 (ge) | changed to reset_parse() which does more reset
+    // reset the parser, including reset filename and unset current chuck *
+    reset_parse();
 
     return ret;
 }
@@ -296,6 +323,19 @@ t_CKBOOL Chuck_Compiler::go( const string & filename, FILE * fd, const char * st
 
 
 
+//-----------------------------------------------------------------------------
+// name: set_file2parse()
+// desc: set a FILE * for one-time use on next go(); see header for details
+//-----------------------------------------------------------------------------
+void Chuck_Compiler::set_file2parse( FILE * fd, t_CKBOOL autoClose )
+{
+    // call down to parser
+    ::fd2parse_set( fd, autoClose );
+}
+
+
+
+
 //-----------------------------------------------------------------------------
 // name: resolve()
 // desc: resolve type automatically - if auto_depend is off, return FALSE
@@ -440,22 +480,27 @@ t_CKBOOL Chuck_Compiler::do_all_except_classes( Chuck_Context * context )
 // name: do_normal_depend()
 // desc: compile normally without auto-depend
 //-----------------------------------------------------------------------------
-t_CKBOOL Chuck_Compiler::do_normal_depend( const string & filename, FILE * fd,
-                                          const char * str_src, const string & full_path )
+t_CKBOOL Chuck_Compiler::do_normal_depend( const string & filename,
+                                           const string & codeLiteral,
+                                           const string & full_path )
 {
     t_CKBOOL ret = TRUE;
     Chuck_Context * context = NULL;
 
+    // expand
+    string theFilename = expand_filepath(filename);
+    string theFullpath = expand_filepath(full_path);
+
     // parse the code
-    if( !chuck_parse( filename.c_str(), fd, str_src ) )
+    if( !chuck_parse( theFilename, codeLiteral ) )
         return FALSE;
 
     // make the context
-    context = type_engine_make_context( g_program, filename );
+    context = type_engine_make_context( g_program, theFilename );
     if( !context ) return FALSE;
 
     // remember full path (added 1.3.0.0)
-    context->full_path = full_path;
+    context->full_path = theFullpath;
 
     // reset the env
     env()->reset();
@@ -494,7 +539,7 @@ cleanup:
     // unload the context from the type-checker
     if( !type_engine_unload_context( env() ) )
     {
-        EM_error2( 0, "internal error unloading context...\n" );
+        EM_error2( 0, "(internal error) unloading context..." );
         return FALSE;
     }
 
@@ -508,14 +553,15 @@ cleanup:
 // name: do_auto_depend()
 // desc: compile with auto-depend (TODO: this doesn't work yet, I don't think)
 //-----------------------------------------------------------------------------
-t_CKBOOL Chuck_Compiler::do_auto_depend( const string & filename, FILE * fd,
-                                         const char * str_src, const string & full_path )
+t_CKBOOL Chuck_Compiler::do_auto_depend( const string & filename,
+                                         const string & codeLiteral,
+                                         const string & full_path )
 {
     t_CKBOOL ret = TRUE;
     Chuck_Context * context = NULL;
 
     // parse the code
-    if( !chuck_parse( filename.c_str(), fd, str_src ) )
+    if( !chuck_parse( filename, codeLiteral ) )
         return FALSE;
 
     // make the context
@@ -538,7 +584,7 @@ t_CKBOOL Chuck_Compiler::do_auto_depend( const string & filename, FILE * fd,
     if( !code )
     {
         ret = FALSE;
-        EM_error2( 0, "internal error: context->code() NULL!" );
+        EM_error2( 0, "(internal error) context->code() NULL!" );
         goto cleanup;
     }
 
@@ -552,7 +598,7 @@ cleanup:
     // unload the context from the type-checker
     if( !type_engine_unload_context( env() ) )
     {
-        EM_error2( 0, "internal error unloading context...\n" );
+        EM_error2( 0, "(internal error) unloading context..." );
         return FALSE;
     }
 
@@ -628,11 +674,10 @@ t_CKBOOL load_module( Chuck_Compiler * compiler, Chuck_Env * env, f_ck_query que
     query_failed = !(dll->load( query ) && dll->query());
     if( query_failed || !type_engine_add_dll( env, dll, nspc ) )
     {
-        CK_FPRINTF_STDERR(
-                 "[chuck]: internal error loading module '%s.%s'...\n",
-                 nspc, name );
+        EM_error2( 0, "internal error loading module '%s.%s'...",
+                   nspc, name );
         if( query_failed )
-            CK_FPRINTF_STDERR( "       %s", dll->last_error() );
+        EM_error2( 0, "...(reason: %s)", dll->last_error() );
 
         return FALSE;
     }
@@ -692,9 +737,6 @@ t_CKBOOL load_internal_modules( Chuck_Compiler * compiler )
     // load
     EM_log( CK_LOG_SEVERE, "module 'machine'" );
     if( !load_module( compiler, env, machine_query, "Machine", "global" ) ) goto error;
-    #ifndef __DISABLE_OTF_SERVER__
-    machine_init( compiler, otf_process_msg );
-    #endif
 
     #ifndef __DISABLE_NETWORK__
     EM_log( CK_LOG_SEVERE, "module 'opsc'" );
@@ -723,16 +765,21 @@ t_CKBOOL load_internal_modules( Chuck_Compiler * compiler )
     // commit what is in the type checker at this point
     env->global()->commit();
 
+    // preserve all operator overloads currently in registry | 1.5.1.5
+    env->op_registry.preserve();
+
     // pop indent level
     EM_poplog();
 
     return TRUE;
 
 error:
-
     // probably dangerous: rollback
     env->global()->rollback();
 
+    // undo op registry entries | 1.5.1.5
+    env->op_registry.reset2local();
+
     // clear context
     type_engine_unload_context( env );
 
@@ -744,6 +791,183 @@ error:
 
 
 
+
+//-----------------------------------------------------------------------------
+// name: getDirEntryAttribute()
+// desc: get directory entry attribute | 1.5.0.4 (ge) moved into function
+//-----------------------------------------------------------------------------
+t_CKBOOL getDirEntryAttribute( dirent * de, t_CKBOOL & isDirectory, t_CKBOOL & isRegular )
+{
+#if defined(__PLATFORM_WINDOWS__)
+    isDirectory = de->data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY;
+    isRegular = ((de->data.dwFileAttributes & FILE_ATTRIBUTE_NORMAL) ||
+                  (de->data.dwFileAttributes & FILE_ATTRIBUTE_READONLY) ||
+                  (de->data.dwFileAttributes & FILE_ATTRIBUTE_ARCHIVE));
+#elif defined(__WINDOWS_PTHREAD__) // Cygwin -- doesn't have dirent d_type
+    std::string absolute_path = std::string(directory) + "/" + de->d_name;
+    struct stat st;
+    if( stat(absolute_path.c_str(), &st) == 0 )
+    {
+        isDirectory = st.st_mode & S_IFDIR;
+        isRegular = st.st_mode & S_IFREG;
+    }
+    else
+    {
+        // uhh ...
+        EM_log( CK_LOG_INFO,
+               "unable to open file '%s', ignoring for chugins",
+               absolute_path.c_str() );
+        return false;
+    }
+#else
+    isDirectory = de->d_type == DT_DIR;
+    isRegular = de->d_type == DT_REG;
+#endif
+
+    return true;
+}
+
+
+
+
+//-----------------------------------------------------------------------------
+// name: struct ChuginFileInfo()
+// desc: file info for loading
+//-----------------------------------------------------------------------------
+struct ChuginFileInfo
+{
+    string filename;
+    string path;
+    bool isBundle;
+
+    // constructor
+    ChuginFileInfo() { isBundle = false;  }
+
+    // constructor
+    ChuginFileInfo( const string & f, const string & p, bool bundle = false )
+    {
+        filename = f; path = p; isBundle = bundle;
+    }
+};
+// compare function for sorting ChuginFileInfo
+static bool comp_func_chuginfileinfo( const ChuginFileInfo & a, const ChuginFileInfo & b )
+{
+    return tolower( a.filename ) < tolower( b.filename );
+}
+
+
+
+
+//-----------------------------------------------------------------------------
+// name: format_dir_name_for_display()
+// desc: format directory name for display, considering underlying platform
+//-----------------------------------------------------------------------------
+static string format_dir_name_for_display( const string & path )
+{
+#ifndef __PLATFORM_WINDOWS__
+    return path;
+#else // __PLATFORM_WINDOWS__
+    return expand_filepath( path );
+#endif
+}
+
+
+
+//-----------------------------------------------------------------------------
+// name: scan_external_modules_in_directory()
+// desc: scan all external modules by extension within a directory
+//-----------------------------------------------------------------------------
+t_CKBOOL scan_external_modules_in_directory( const char * directory,
+    const char * extension, t_CKBOOL recursiveSearch,
+    vector<ChuginFileInfo> & chugins2load, vector<string> & dirs2search,
+    vector<string> & ckfiles2load )
+{
+    // expand directory path
+    string path = expand_filepath( string(directory), FALSE );
+    // open the directory
+    DIR * dir = opendir( path.c_str() );
+
+    // cannot open
+    if( !dir ) return FALSE;
+
+    // do first read | 1.5.0.0 (ge + eito) #chunreal
+    struct dirent * de = readdir( dir );
+    // while( (de = readdir(dir)) ) <- UE5 forces us to not do this
+    while( de != NULL )
+    {
+        t_CKBOOL is_regular = false;
+        t_CKBOOL is_directory = false;
+        // get attributes
+        if( !getDirEntryAttribute( de, is_directory, is_regular ) ) continue;
+
+        // check | TODO: follow links?
+        if( is_regular )
+        {
+            // check for ck files to pre-load
+            if( extension_matches( de->d_name, ".ck" ) )
+            {
+                // construct absolute path
+                std::string absolute_path = path + "/" + de->d_name;
+                // append file path
+                ckfiles2load.push_back( absolute_path );
+            }
+            // check extension passed in
+            else if( extension_matches( de->d_name, extension ) )
+            {
+                // construct absolute path
+                std::string absolute_path = path + "/" + de->d_name;
+                // append module
+                chugins2load.push_back( ChuginFileInfo( de->d_name, absolute_path ) );
+            }
+        }
+        // recursive search? into directories...
+        // (whose name doesn't end in .chug, unless it was precisely .chug)
+        else if( recursiveSearch && is_directory && subdir_ok2recurse( de->d_name, extension ) )
+        {
+            // recurse | TODO: max depth?
+            if( strncmp( de->d_name, ".", sizeof( "." ) ) != 0 &&
+                strncmp( de->d_name, "..", sizeof( ".." ) ) != 0 )
+            {
+                // construct absolute path (use the non-expanded path for consistency when printing)
+                std::string absolute_path = string(directory) + "/" + de->d_name;
+                // queue search in sub-directory
+                dirs2search.push_back( absolute_path );
+            }
+        }
+#ifdef __PLATFORM_APPLE__
+        else if( is_directory && extension_matches( de->d_name, extension ) )
+        {
+            // On macOS, a chugin can either be a regular file (a .dylib simply renamed to .chug)
+            // or it can be a MODULE like using CMake with add_library(FooChugin MODULE foo.cpp)
+            // If it's a module bundle, then because .chug is a nonstandard extension,
+            // on the filesystem it shows up as a directory ending in .chug.
+            // If we see one of these directories, we can dive directly to the Contents/MacOS subfolder
+            // and look for a regular file. | 1.5.0.1 (dbraun) added
+            std::string absolute_path = path + "/" + de->d_name + "/Contents/MacOS";
+            // probe NAME.chug/Contents/MacOS/NAME
+            string actualName = extension_removed( de->d_name, extension );
+            // construct
+            absolute_path += "/" + actualName;
+            // get cstr
+            const char * subdirectory = absolute_path.c_str();
+            // load directly
+            chugins2load.push_back( ChuginFileInfo( de->d_name, subdirectory, true ) );
+        }
+#endif // #ifdef __PLATFORM_APPLE__
+
+        // read next | 1.5.0.0 (ge) moved here due to #chunreal
+        de = readdir( dir );
+    }
+
+    // close
+    closedir( dir );
+
+    return TRUE;
+}
+
+
+
+
 //-----------------------------------------------------------------------------
 // name: load_external_module_at_path()
 // desc: ...
@@ -754,49 +978,77 @@ t_CKBOOL load_external_module_at_path( Chuck_Compiler * compiler,
 {
     Chuck_Env * env = compiler->env();
 
-    EM_log(CK_LOG_SEVERE, "loading chugin '%s'", name);
+    // EM_log(CK_LOG_SEVERE, "loading chugin '%s'", name);
 
     Chuck_DLL * dll = new Chuck_DLL( compiler->carrier(), name );
     t_CKBOOL query_failed = FALSE;
 
-    // try to load and query
-    query_failed = !(dll->load(dl_path) && dll->query());
-    if( query_failed || !type_engine_add_dll2(env, dll, "global"))
+    // load (but don't query yet; lazy mode == TRUE)
+    if( dll->load(dl_path, CK_QUERY_FUNC, TRUE) )
     {
-        EM_pushlog();
-        EM_log(CK_LOG_SEVERE, "cannot load chugin '%s', skipping...", name);
-        // EM_error2( 0, "error: cannot load chugin '%s', skipping...", name );
-        if( query_failed )
+        // probe it
+        dll->probe();
+        // if not compatible
+        if( !dll->compatible() )
         {
-            // EM_error2( 0, "%s", dll->last_error() );
-            EM_log( CK_LOG_SEVERE, "%s", dll->last_error() );
+            // print
+            EM_log( CK_LOG_SEVERE, "[%s] loading chugin %s (%d.%d)", TC::red("FAILED",true).c_str(), name, dll->versionMajor(), dll->versionMinor() );
+            // push
+            EM_pushlog();
+            EM_log( CK_LOG_SEVERE, "reason: %s", TC::orange(dll->last_error(),true).c_str() );
+            EM_poplog();
+            // go to error for cleanup
+            goto error;
         }
-        delete dll;
-        EM_poplog();
 
-        return FALSE;
+        // try to query the chugin
+        query_failed = !dll->query();
+        // try to add to type system
+        if( query_failed || !type_engine_add_dll2( env, dll, "global" ) )
+        {
+            // print
+            EM_log( CK_LOG_SEVERE, "[%s] loading chugin %s (%d.%d)", TC::red("FAILED",true).c_str(), name, dll->versionMajor(), dll->versionMinor() );
+            EM_pushlog();
+            // if add_dll2 failed, an error should have already been output
+            if( query_failed )
+            {
+                // print reason
+                EM_log( CK_LOG_SEVERE, "reason: %s", TC::orange(dll->last_error(),true).c_str() );
+            }
+            EM_log( CK_LOG_SEVERE, "%s '%s'...", TC::blue("skipping",true).c_str(), dl_path );
+            EM_poplog();
+            // go to error for cleanup
+            goto error;
+        }
     }
     else
     {
-        compiler->m_dlls.push_back(dll);
-        return TRUE;
+        // print
+        EM_log( CK_LOG_SEVERE, "[%s] chugin '%s' load...", TC::red("FAILED",true).c_str(), name );
+        // more info
+        EM_pushlog();
+        EM_log( CK_LOG_SEVERE, "reason: %s", TC::orange(dll->last_error(),true).c_str() );
+        EM_poplog();
+        // go to error for cleanup
+        goto error;
     }
-}
 
+    // print
+    EM_log( CK_LOG_SEVERE, "[%s] chugin %s (%d.%d)", TC::green("OK",true).c_str(), name, dll->versionMajor(), dll->versionMinor() );
+    // add to compiler
+    compiler->m_dlls.push_back(dll);
+    // commit operator overloads | 1.5.1.5
+    compiler->env()->op_registry.preserve();
+    // return home successful
+    return TRUE;
 
+error:
+    // clean up
+    CK_SAFE_DELETE( dll );
+    // rollback operator overloads | 1.5.1.5
+    compiler->env()->op_registry.reset2local();
 
-
-//-----------------------------------------------------------------------------
-// name: extension_matches()
-// desc: ...
-//-----------------------------------------------------------------------------
-static t_CKBOOL extension_matches( const char * filename, const char * extension )
-{
-    t_CKUINT extension_length = strlen(extension);
-    t_CKUINT filename_length = strlen(filename);
-
-    return strncmp( extension, filename+(filename_length-extension_length),
-                    extension_length) == 0;
+    return FALSE;
 }
 
 
@@ -804,142 +1056,74 @@ static t_CKBOOL extension_matches( const char * filename, const char * extension
 
 //-----------------------------------------------------------------------------
 // name: load_external_modules_in_directory()
-// desc: ...
+// desc: load all external modules by extension within a directory
 //-----------------------------------------------------------------------------
 t_CKBOOL load_external_modules_in_directory( Chuck_Compiler * compiler,
                                              const char * directory,
-                                             const char * extension )
+                                             const char * extension,
+                                             t_CKBOOL recursiveSearch )
 {
-    static const t_CKBOOL RECURSIVE_SEARCH = false;
+    vector<ChuginFileInfo> chugins2load;
+    vector<string> dirs2search;
+    vector<string> ckfiles2load;
 
-    // directory path
-    string path = directory, exp = "";
+    // print directory to examine
+    EM_log( CK_LOG_SEVERE, "searching '%s'", format_dir_name_for_display(directory).c_str() );
+    // push
+    EM_pushlog();
 
-#ifndef __PLATFORM_WIN32__
-#ifndef __DISABLE_WORDEXP__
-    // 1.5.0.0 (ge) added (for ~)
-    wordexp_t exp_result;
-    // "perform shell-style word expansions"
-    wordexp( directory, &exp_result, 0 );
-    // concatenate in case there are spaces in the path
-    for( t_CKINT i = 0; i < exp_result.we_wordc; i++ )
-        exp += string(i > 0 ? " " : "") + exp_result.we_wordv[i];
-    // replace
-    path = exp;
-#endif
-#endif
-
-    // open the directory
-    DIR * dir = opendir( path.c_str() );
-
-    // if successful
-    if( dir )
-    {
-        // log
-        EM_log( CK_LOG_INFO, "examining directory '%s' for chugins", directory );
-
-        // do first read | 1.5.0.0 (ge + eito) #chunreal
-        struct dirent * de = readdir(dir);
-
-        // while( (de = readdir(dir)) ) <- in ge's opinion clearer than what UE5 forces us to do
-        while( de != NULL )
-        {
-            t_CKBOOL is_regular = false;
-            t_CKBOOL is_directory = false;
-
-#if defined(__PLATFORM_WIN32__)
-            is_directory = de->data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY;
-            is_regular = ((de->data.dwFileAttributes & FILE_ATTRIBUTE_NORMAL) ||
-                          (de->data.dwFileAttributes & FILE_ATTRIBUTE_READONLY) ||
-                          (de->data.dwFileAttributes & FILE_ATTRIBUTE_ARCHIVE));
-#elif defined(__WINDOWS_PTHREAD__) // Cygwin -- doesn't have dirent d_type
-            std::string absolute_path = std::string(directory) + "/" + de->d_name;
-            struct stat st;
-            if( stat(absolute_path.c_str(), &st) == 0 )
-            {
-                is_directory = st.st_mode & S_IFDIR;
-                is_regular = st.st_mode & S_IFREG;
-            }
-            else
-            {
-                // uhh ...
-                EM_log( CK_LOG_INFO,
-                        "unable to open file '%s', ignoring for chugins",
-                        absolute_path.c_str() );
-                continue;
-            }
-#else
-            is_directory = de->d_type == DT_DIR;
-            is_regular = de->d_type == DT_REG;
-#endif
-
-            if( is_regular ) // TODO: follow links?
-            {
-                if( extension_matches(de->d_name, extension) )
-                {
-                    std::string absolute_path = std::string(directory) + "/" + de->d_name;
-
-                    load_external_module_at_path(compiler, de->d_name,
-                                                 absolute_path.c_str());
-                }
-                else if( extension_matches(de->d_name, ".ck") )
-                {
-                    std::string absolute_path = std::string(directory) + "/" + de->d_name;
-                    compiler->m_cklibs_to_preload.push_back(absolute_path);
-                }
-            }
-            else if( RECURSIVE_SEARCH && is_directory )
-            {
-                // recurse
-                // TODO: max depth?
-                if( strncmp(de->d_name, ".", sizeof(".")) != 0 &&
-                    strncmp(de->d_name, "..", sizeof("..")) != 0 )
-                {
-                    std::string absolute_path = std::string(directory) + "/" + de->d_name;
-                    load_external_modules_in_directory(compiler,
-                                                       absolute_path.c_str(),
-                                                       extension);
-                }
-            }
-#ifdef __PLATFORM_MACOSX__
-            // #if __APPLE__ probably cleaner but using the above for consistency for now (ge)
-            // ---
-            // On macOS, a chugin can either be a regular file (a .dylib simply renamed to .chug)
-            // or it can be a MODULE like using CMake with add_library(FooChugin MODULE foo.cpp)
-            // If it's a module bundle, then because .chug is a nonstandard extension,
-            // on the filesystem it shows up as a directory ending in .chug.
-            // If we see one of these directories, we can dive directly to the Contents/MacOS subfolder
-            // and look for a regular file. | 1.5.0.1 (dbraun) added
-            else if (is_directory && extension_matches(de->d_name, extension)) {
-                std::string absolute_path = std::string(directory) + "/" + de->d_name + "/Contents/MacOS";
-                const char* subdirectory = absolute_path.c_str();
-                load_external_modules_in_directory(compiler, subdirectory, "");
-            }
-#endif // #ifdef __PLATFORM_MACOSX__
-
-            // read next | 1.5.0.0 (ge) moved here due to #chunreal
-            de = readdir( dir );
-        }
-
-        // close
-        closedir( dir );
-    }
-    else
+    // scan
+    t_CKBOOL retval = scan_external_modules_in_directory( directory, extension, recursiveSearch, chugins2load, dirs2search, ckfiles2load );
+    if( !retval )
     {
         // log (1.3.1.2: changed to 2 lines to stay within 80 chars)
-        EM_log( CK_LOG_INFO, "unable to open directory:" );
-        EM_pushlog();
-        EM_log( CK_LOG_INFO, "'%s'", directory );
+        EM_log( CK_LOG_INFO, "unable to open directory..." );
         EM_log( CK_LOG_INFO, "ignoring for chugins..." );
+        // pop
         EM_poplog();
+        // go ahead and return true
+        return TRUE;
     }
 
-#ifndef __PLATFORM_WIN32__
-#ifndef __DISABLE_WORDEXP__
-    // free | 1.5.0.0 (ge) added
-    wordfree( &exp_result );
-#endif
-#endif
+    // sort
+    sort( chugins2load.begin(), chugins2load.end(), comp_func_chuginfileinfo );
+    sort( dirs2search.begin(), dirs2search.end() );
+    sort( ckfiles2load.begin(), ckfiles2load.end() );
+
+    // loop over chugins to load
+    for( t_CKINT i = 0; i < chugins2load.size(); i++ )
+    {
+        // load module
+        t_CKBOOL loaded = load_external_module_at_path( compiler, chugins2load[i].filename.c_str(),
+            chugins2load[i].path.c_str() );
+        // if no error
+        if( chugins2load[i].isBundle && loaded) {
+            // log
+            EM_pushlog();
+            EM_log( CK_LOG_INFO, "macOS bundle was detected..." );
+            string shortenSubdir = chugins2load[i].path;
+            shortenSubdir = shortenSubdir.substr( shortenSubdir.find( chugins2load[i].filename ) );
+            EM_log( CK_LOG_INFO, "loaded %s", shortenSubdir.c_str() );
+            EM_poplog();
+        }
+    }
+
+    // loop over ck files to load (later)
+    for( t_CKINT i = 0; i < ckfiles2load.size(); i++ )
+    {
+        // save for later
+        compiler->m_cklibs_to_preload.push_back( ckfiles2load[i] );
+    }
+
+    // pop log
+    EM_poplog();
+
+    // loop over dirs2 to search | if not recursive, this should be empty
+    for( t_CKINT i = 0; i < dirs2search.size(); i++ )
+    {
+        // search in dir
+        load_external_modules_in_directory( compiler, dirs2search[i].c_str(), extension, recursiveSearch );
+    }
 
     return TRUE;
 }
@@ -949,11 +1133,12 @@ t_CKBOOL load_external_modules_in_directory( Chuck_Compiler * compiler,
 
 //-----------------------------------------------------------------------------
 // name: load_external_modules()
-// desc: ...
+// desc: load external modules (e.g., .chug and .ck library files)
 //-----------------------------------------------------------------------------
-t_CKBOOL Chuck_Compiler:: load_external_modules( const char * extension,
-    std::list<std::string> & chugin_search_paths,
-    std::list<std::string> & named_dls )
+t_CKBOOL Chuck_Compiler::load_external_modules( const char * extension,
+                                                std::list<std::string> & chugin_search_paths,
+                                                std::list<std::string> & named_dls,
+                                                t_CKBOOL recursiveSearch )
 {
     // get env
     Chuck_Env * env = this->env();
@@ -964,44 +1149,213 @@ t_CKBOOL Chuck_Compiler:: load_external_modules( const char * extension,
     // load it
     type_engine_load_context( env, context );
 
-    /* first load dynamic libraries explicitly named on the command line */
-    for(std::list<std::string>::iterator i_dl = named_dls.begin();
-        i_dl != named_dls.end(); i_dl++)
+    // first load dynamic libraries explicitly named on the command line
+    for( std::list<std::string>::iterator i_dl = named_dls.begin();
+         i_dl != named_dls.end(); i_dl++ )
     {
+        // get chugin name
         std::string & dl_path = *i_dl;
-        if(!extension_matches(dl_path.c_str(), extension))
+        // check extension, append if no match
+        if( !extension_matches(dl_path.c_str(), extension) )
             dl_path += extension;
-
-        load_external_module_at_path(this, dl_path.c_str(),
-                                     dl_path.c_str());
+        // load the module
+        load_external_module_at_path( this, dl_path.c_str(), dl_path.c_str() );
     }
 
-    /* now recurse through search paths and load any DLs or .ck files found */
-    for(std::list<std::string>::iterator i_sp = chugin_search_paths.begin();
-        i_sp != chugin_search_paths.end(); i_sp++)
+    // now recurse through search paths and load any DLs or .ck files found
+    for( std::list<std::string>::iterator i_sp = chugin_search_paths.begin();
+         i_sp != chugin_search_paths.end(); i_sp++ )
     {
-        load_external_modules_in_directory(this, (*i_sp).c_str(), extension);
+        // search directory and load contents
+        load_external_modules_in_directory( this, (*i_sp).c_str(), extension, recursiveSearch );
     }
 
     // clear context
     type_engine_unload_context( env );
-
     // commit what is in the type checker at this point
     env->global()->commit();
+    // preserve all operator overloads currently in registry | 1.5.1.5
+    env->op_registry.preserve();
 
     return TRUE;
-   /*
-error:
+}
 
-    // probably dangerous: rollback
-    env->global()->rollback();
 
-    // clear context
-    type_engine_unload_context( env );
 
-    // pop indent level
+
+//-----------------------------------------------------------------------------
+// name: probe_external_module_at_path()
+// desc: probe external module
+//-----------------------------------------------------------------------------
+t_CKBOOL probe_external_module_at_path( const char * name, const char * dl_path )
+{
+    // create dynamic module
+    Chuck_DLL * dll = new Chuck_DLL( NULL, name );
+
+    // load the dll, lazy mode
+    if( dll->load(dl_path, CK_QUERY_FUNC, TRUE) )
+    {
+        // probe it
+        dll->probe();
+        // if not compatible
+        if( dll->compatible() )
+        {
+            // print
+            EM_log( CK_LOG_SYSTEM, "[%s] chugin %s (%d.%d)", TC::green("OK",true).c_str(), name, dll->versionMajor(), dll->versionMinor() );
+        }
+        else
+        {
+            // print
+            EM_log( CK_LOG_SYSTEM, "[%s] chugin %s (%d.%d)", TC::red("FAILED",true).c_str(), name, dll->versionMajor(), dll->versionMinor() );
+            // push
+            EM_pushlog();
+            EM_log( CK_LOG_SYSTEM, "reason: %s", TC::orange(dll->last_error(),true).c_str() );
+            EM_poplog();
+
+        }
+    }
+    else
+    {
+        // print
+        EM_log( CK_LOG_SYSTEM, "[%s] chugin '%s' load...", TC::red("FAILED",true).c_str(), name );
+        // more info
+        EM_pushlog();
+        EM_log( CK_LOG_SYSTEM, "reason: %s", TC::orange(dll->last_error(),true).c_str() );
+        EM_poplog();
+    }
+
+    // done
+    return TRUE;
+}
+
+
+
+//-----------------------------------------------------------------------------
+// name: probe_external_modules_in_directory()
+// desc: probe all external modules by extension within a directory
+//-----------------------------------------------------------------------------
+t_CKBOOL probe_external_modules_in_directory( const char * directory,
+                                              const char * extension,
+                                              t_CKBOOL recursiveSearch,
+                                              std::list<std::string> & ck_libs )
+{
+    vector<ChuginFileInfo> chugins2load;
+    vector<string> dirs2search;
+    vector<string> ckfiles2load;
+
+    // print directory to examine
+    EM_log( CK_LOG_SYSTEM, "searching '%s'", format_dir_name_for_display( directory ).c_str() );
+    // push
+    EM_pushlog();
+
+    // scan
+    t_CKBOOL retval = scan_external_modules_in_directory( directory, extension, recursiveSearch, chugins2load, dirs2search, ckfiles2load );
+    if( !retval )
+    {
+        // log (1.3.1.2: changed to 2 lines to stay within 80 chars)
+        EM_log( CK_LOG_INFO, "unable to open directory..." );
+        EM_log( CK_LOG_INFO, "ignoring for chugins..." );
+        // pop log
+        EM_poplog();
+        // go ahead and return true
+        return TRUE;
+    }
+
+    // sort
+    sort( chugins2load.begin(), chugins2load.end(), comp_func_chuginfileinfo );
+    sort( dirs2search.begin(), dirs2search.end() );
+    sort( ckfiles2load.begin(), ckfiles2load.end() );
+
+    // loop over chugins to load
+    for( t_CKINT i = 0; i < chugins2load.size(); i++ )
+    {
+        // load module
+        t_CKBOOL probed = probe_external_module_at_path(
+            chugins2load[i].filename.c_str(), chugins2load[i].path.c_str() );
+        // if no error
+        if( chugins2load[i].isBundle && probed) {
+            // log
+            EM_pushlog();
+            EM_log( CK_LOG_INFO, "macOS bundle was detected..." );
+            string shortenSubdir = chugins2load[i].path;
+            shortenSubdir = shortenSubdir.substr( shortenSubdir.find( chugins2load[i].filename ) );
+            EM_log( CK_LOG_INFO, "loaded %s", shortenSubdir.c_str() );
+            EM_poplog();
+        }
+    }
+
+    // loop over ck files to load (later)
+    for( t_CKINT i = 0; i < ckfiles2load.size(); i++ )
+    {
+        // save for later
+        ck_libs.push_back( ckfiles2load[i] );
+    }
+
+    // pop log
     EM_poplog();
 
-    return FALSE;
-   */
+    // loop over dirs2 to search | if not recursive, this should be empty
+    for( t_CKINT i = 0; i < dirs2search.size(); i++ )
+    {
+        // search in dir
+        probe_external_modules_in_directory( dirs2search[i].c_str(), extension, recursiveSearch, ck_libs );
+    }
+
+    return TRUE;
+}
+
+
+
+
+//-----------------------------------------------------------------------------
+// name: probe_external_modules()
+// desc: probe external modules (e.g., .chug and .ck library files)
+//-----------------------------------------------------------------------------
+t_CKBOOL Chuck_Compiler::probe_external_modules( const char * extension,
+                                                 std::list<std::string> & chugin_search_paths,
+                                                 std::list<std::string> & named_dls,
+                                                 t_CKBOOL recursiveSearch,
+                                                 std::list<std::string> & ck_libs )
+{
+    // print
+    EM_log( CK_LOG_SYSTEM, "probing specified chugins (e.g., via --chugin)..." );
+    // push
+    EM_pushlog();
+
+    // first load dynamic libraries explicitly named on the command line
+    for( std::list<std::string>::iterator i_dl = named_dls.begin();
+         i_dl != named_dls.end(); i_dl++ )
+    {
+        // get chugin name
+        std::string & dl_path = *i_dl;
+        // check extension, append if no match
+        if( !extension_matches(dl_path.c_str(), extension) )
+            dl_path += extension;
+        // load the module
+        probe_external_module_at_path( dl_path.c_str(), dl_path.c_str() );
+    }
+
+    // check
+    if( named_dls.size() == 0 )
+        EM_log( CK_LOG_INFO, "(none specified)" );
+
+    // pop
+    EM_poplog();
+    // print
+    EM_log( CK_LOG_SYSTEM, "probing chugins in search paths..." );
+    // push
+    EM_pushlog();
+
+    // now recurse through search paths and load any DLs or .ck files found
+    for( std::list<std::string>::iterator i_sp = chugin_search_paths.begin();
+         i_sp != chugin_search_paths.end(); i_sp++ )
+    {
+        // search directory and load contents
+        probe_external_modules_in_directory( (*i_sp).c_str(), extension, recursiveSearch, ck_libs );
+    }
+
+    // pop
+    EM_poplog();
+
+    return TRUE;
 }
