@@ -97,7 +97,7 @@ static const char * g_str[] = {
     "NONE",         // 0
     "CKCORE",       // 1
     "SYSTEM",       // 2
-    "SEVERE",       // 3
+    "HERALD",       // 3 <-- 1.5.1.3 changed from "SEVERE" to "HERALD"
     "WARN!!",       // 4
     "INFORM",       // 5
     "DEBUG!",       // 6 <-- 1.5.0.5 changed from "CONFIG" to "DEBUG!"
@@ -258,7 +258,7 @@ void EM_printLineInCode( t_CKINT lineNumber, t_CKINT charNumber )
         line = replace_tabs( line, ' ' );
 
         // line position
-        string posLine = itoa(lineNumber);
+        string posLine = ck_itoa(lineNumber);
         // spaces due to lineNumber + [] + space
         t_CKINT posLineIndent = posLine.length() + 3;
 
@@ -435,7 +435,7 @@ void EM_error( t_CKINT pos, const char * message, ... )
     // if found
     if( pos > 0 && actualPos > 0 )
     {
-        CK_FPRINTF_STDERR( "%s%s%s%s", COLON.c_str(), TC::blue(itoa(line),bold).c_str(), COLON.c_str(), TC::green(itoa(actualPos),bold).c_str() );
+        CK_FPRINTF_STDERR( "%s%s%s%s", COLON.c_str(), TC::blue(ck_itoa(line),bold).c_str(), COLON.c_str(), TC::green(ck_itoa(actualPos),bold).c_str() );
         snprintf( g_buffer, CK_ERR_BUF_LENGTH, ":%d:%d", (int)line, (int)actualPos );
         lastErrorCat( g_buffer );
     }
@@ -507,7 +507,7 @@ void EM_error2( t_CKINT pos, const char * message, ... )
     // if given a valid line number
     if( pos )
     {
-        CK_FPRINTF_STDERR( "%s%s%s%s", COLON.c_str(), TC::blue(itoa(line),bold).c_str(), COLON.c_str(), TC::green(itoa(actualPos),bold).c_str() );
+        CK_FPRINTF_STDERR( "%s%s%s%s", COLON.c_str(), TC::blue(ck_itoa(line),bold).c_str(), COLON.c_str(), TC::green(ck_itoa(actualPos),bold).c_str() );
         snprintf( g_buffer, CK_ERR_BUF_LENGTH, ":%d:%d", (int)line, (int)actualPos );
         lastErrorCat( g_buffer );
     }
@@ -516,9 +516,22 @@ void EM_error2( t_CKINT pos, const char * message, ... )
 
     if( pos )
     {
+        // check for special message types | 1.5.1.3
+        const char * msgType = "error: ";
+        // copy into c++ string
+        string msg = message;
+        // look for special strings
+        if( msg.find("deprecated: ") == 0 )
+        {
+            // update msgType
+            msgType = "deprecated: ";
+            // skip the msgType
+            message += string(msgType).length();
+        }
+
         // print error only if there is non-zero line number
-        CK_FPRINTF_STDERR( "%s", TC::red("error: ", true).c_str() );
-        lastErrorCat( "error: " );
+        CK_FPRINTF_STDERR( "%s", TC::red(msgType,true).c_str() );
+        lastErrorCat( msgType );
     }
 
     va_start( ap, message );
@@ -613,6 +626,32 @@ void EM_error3( const char * message, ... )
 
 
 
+//-----------------------------------------------------------------------------
+// name: EM_exception()
+// desc: prints exception message
+//-----------------------------------------------------------------------------
+void EM_exception( const char * message, ... )
+{
+    va_list ap;
+
+    // preamble
+    CK_FPRINTF_STDERR( "[%s]:(%s) ",
+                       TC::orange("chuck",true).c_str(),
+                       TC::red("EXCEPTION",true).c_str() );
+
+    // actual exception message text
+    va_start( ap, message );
+    CK_VFPRINTF_STDERR( TC::orange(message,true).c_str(), ap );
+    va_end( ap );
+
+    // end line and flush to be extra sure
+    CK_FPRINTF_STDERR( "\n" );
+    CK_FFLUSH_STDERR();
+}
+
+
+
+
 //-----------------------------------------------------------------------------
 // name: EM_print2vanilla()
 // format: like EM_error() minus the error and the line
@@ -759,7 +798,7 @@ void EM_log( t_CKINT level, const char * message, ... )
     TC::off();
     CK_FPRINTF_STDERR( "[%s:%s:%s]: ",
                        TC::blue("chuck", true).c_str(),
-                       TC::blue( itoa(level), false ).c_str(),
+                       TC::blue( ck_itoa(level), false ).c_str(),
                        TC::blue( g_str[level] ).c_str() );
     TC::on();
 
@@ -1219,17 +1258,22 @@ ChuckOutStream::~ChuckOutStream()
 {
 }
 
-ChuckOutStream& ChuckOutStream::operator<<( const std::string val )
+ChuckOutStream& ChuckOutStream::operator<<( const std::string & val )
 {
     m_stream << val;
-    if( m_isErr || (val == CK_STDENDL) ) { this->flush(); }
+    if( m_isErr || (val == CK_STDENDL)
+        /* || val.find("\n") != std::string::npos */ )
+    { this->flush(); }
     return *this;
 }
 
 ChuckOutStream& ChuckOutStream::operator<<( const char * val )
 {
     m_stream << val;
-    if( m_isErr || std::string(val) == CK_STDENDL ) { this->flush(); }
+    string s = val;
+    if( m_isErr || s == CK_STDENDL
+        /* || s.find("\n") != std::string::npos */ )
+    { this->flush(); }
     return *this;
 }
 
