@@ -53,9 +53,7 @@ t_CKBOOL init_class_MidiRW( Chuck_Env * env );
 t_CKBOOL init_class_HID( Chuck_Env * env );
 #endif
 t_CKBOOL init_class_io( Chuck_Env * env, Chuck_Type * type );
-// #ifndef __DISABLE_FILEIO__
 t_CKBOOL init_class_fileio( Chuck_Env * env, Chuck_Type * type );
-// #endif
 // added 1.3.0.0 -- moved to be full-fledged class
 t_CKBOOL init_class_chout( Chuck_Env * env, Chuck_Type * type );
 // added 1.3.0.0 -- moved to be full-fledged class
@@ -100,6 +98,14 @@ public:
     virtual void write( t_CKINT val, t_CKINT flags ) = 0;
     virtual void write( t_CKFLOAT val ) = 0;
     virtual void write( t_CKFLOAT val, t_CKINT flags ) = 0;
+    virtual void write( const t_CKCOMPLEX & val ) = 0;
+    virtual void write( const t_CKCOMPLEX & val, t_CKINT flags ) = 0;
+    virtual void write( const t_CKPOLAR & val ) = 0;
+    virtual void write( const t_CKPOLAR & val, t_CKINT flags ) = 0;
+    virtual void write( const t_CKVEC3 & val ) = 0;
+    virtual void write( const t_CKVEC3 & val, t_CKINT flags ) = 0;
+    virtual void write( const t_CKVEC4 & val ) = 0;
+    virtual void write( const t_CKVEC4 & val, t_CKINT flags ) = 0;
 
 public:
     // type | moved to IO in 1.5.0.0 (ge)
@@ -150,7 +156,6 @@ public:
 
 
 
-// #ifndef __DISABLE_FILEIO__
 
 #ifndef __PLATFORM_WINDOWS__
 #include <dirent.h>
@@ -185,7 +190,7 @@ public:
 
     // directories
     virtual t_CKINT isDir();
-    virtual Chuck_Array4 * dirList();
+    virtual Chuck_ArrayInt * dirList();
 
     // reading
     // virtual Chuck_String * read( t_CKINT length );
@@ -210,6 +215,14 @@ public:
     virtual void write( t_CKINT val, t_CKINT flags );
     virtual void write( t_CKFLOAT val );
     virtual void write( t_CKFLOAT val, t_CKINT flags );
+    virtual void write( const t_CKCOMPLEX & val );
+    virtual void write( const t_CKCOMPLEX & val, t_CKINT flags );
+    virtual void write( const t_CKPOLAR & val );
+    virtual void write( const t_CKPOLAR & val, t_CKINT flags );
+    virtual void write( const t_CKVEC3 & val );
+    virtual void write( const t_CKVEC3 & val, t_CKINT flags );
+    virtual void write( const t_CKVEC4 & val );
+    virtual void write( const t_CKVEC4 & val, t_CKINT flags );
 
 #ifndef __DISABLE_THREADS__
     // writing -- async
@@ -218,6 +231,9 @@ public:
     static THREAD_RETURN( THREAD_TYPE writeFloat_thread ) (void * data);
 #endif
 
+    // get filename
+    std::string filename() const { return m_path; }
+
 protected:
     // open flags
     t_CKINT m_flags;
@@ -233,8 +249,13 @@ protected:
     std::string m_path;
     // vm and shred
     Chuck_VM * m_vmRef;
+
+public:
+    // auto prefix | 1.5.1.5
+    std::string m_autoPrefix;
+    // auto extension | 1.5.1.5
+    std::string m_autoExtension;
 };
-// #endif // __DISABLE_FILEIO__
 
 
 
@@ -271,6 +292,14 @@ public:
     virtual void write( t_CKINT val, t_CKINT flags );
     virtual void write( t_CKFLOAT val );
     virtual void write( t_CKFLOAT val, t_CKINT flags );
+    virtual void write( const t_CKCOMPLEX & val );
+    virtual void write( const t_CKCOMPLEX & val, t_CKINT flags );
+    virtual void write( const t_CKPOLAR & val );
+    virtual void write( const t_CKPOLAR & val, t_CKINT flags );
+    virtual void write( const t_CKVEC3 & val );
+    virtual void write( const t_CKVEC3 & val, t_CKINT flags );
+    virtual void write( const t_CKVEC4 & val );
+    virtual void write( const t_CKVEC4 & val, t_CKINT flags );
 
 public: // REFACTOR-2017
     // set callback
@@ -318,6 +347,14 @@ public:
     virtual void write( t_CKINT val, t_CKINT flags );
     virtual void write( t_CKFLOAT val );
     virtual void write( t_CKFLOAT val, t_CKINT flags );
+    virtual void write( const t_CKCOMPLEX & val );
+    virtual void write( const t_CKCOMPLEX & val, t_CKINT flags );
+    virtual void write( const t_CKPOLAR & val );
+    virtual void write( const t_CKPOLAR & val, t_CKINT flags );
+    virtual void write( const t_CKVEC3 & val );
+    virtual void write( const t_CKVEC3 & val, t_CKINT flags );
+    virtual void write( const t_CKVEC4 & val );
+    virtual void write( const t_CKVEC4 & val, t_CKINT flags );
 
 public:
     // set callback | REFACTOR-2017
@@ -344,7 +381,6 @@ CK_DLL_SFUN( io_newline );
 CK_DLL_SFUN( io_openfile );
 
 
-// #ifndef __DISABLE_FILEIO__
 //-----------------------------------------------------------------------------
 // fileio API
 //-----------------------------------------------------------------------------
@@ -378,7 +414,11 @@ CK_DLL_MFUN( fileio_writeint );
 CK_DLL_MFUN( fileio_writeintflags );
 CK_DLL_MFUN( fileio_writefloat );
 CK_DLL_MFUN( fileio_writefloatflags );
-// #endif // __DISABLE_FILEIO__
+CK_DLL_MFUN( file_ctrl_autoPrefixAndExtension );
+CK_DLL_MFUN( file_cget_autoPrefix );
+CK_DLL_MFUN( file_cget_autoExtension );
+CK_DLL_MFUN( file_cget_filename );
+CK_DLL_SFUN( fileio_expandpath_impl );
 
 
 //-----------------------------------------------------------------------------
@@ -518,8 +558,11 @@ CK_DLL_MFUN( HidIn_open );
 CK_DLL_MFUN( HidIn_open_named );
 CK_DLL_MFUN( HidIn_open_named_i ); // added 1.3.0.0
 CK_DLL_MFUN( HidIn_open_joystick );
+CK_DLL_MFUN( HidIn_open_joystick_2 );
 CK_DLL_MFUN( HidIn_open_mouse );
+CK_DLL_MFUN( HidIn_open_mouse_2 );
 CK_DLL_MFUN( HidIn_open_keyboard );
+CK_DLL_MFUN( HidIn_open_keyboard_2 );
 CK_DLL_MFUN( HidIn_open_tiltsensor );
 CK_DLL_MFUN( HidIn_good );
 CK_DLL_MFUN( HidIn_num );
@@ -589,9 +632,17 @@ public:
     virtual void write( const std::string & val );
     virtual void write( t_CKINT val );
     virtual void write( t_CKINT val, t_CKINT flags );
-    virtual void writeBytes( Chuck_Array4 * arr );
+    virtual void writeBytes( Chuck_ArrayInt * arr );
     virtual void write( t_CKFLOAT val );
     virtual void write( t_CKFLOAT val, t_CKINT flags );
+    virtual void write( const t_CKCOMPLEX & val );
+    virtual void write( const t_CKCOMPLEX & val, t_CKINT flags );
+    virtual void write( const t_CKPOLAR & val );
+    virtual void write( const t_CKPOLAR & val, t_CKINT flags );
+    virtual void write( const t_CKVEC3 & val );
+    virtual void write( const t_CKVEC3 & val, t_CKINT flags );
+    virtual void write( const t_CKVEC4 & val );
+    virtual void write( const t_CKVEC4 & val, t_CKINT flags );
 
     // serial stuff
     virtual t_CKBOOL setBaudRate( t_CKUINT rate );
