@@ -47,7 +47,7 @@ using namespace std;
 //-----------------------------------------------------------------------------
 t_CKBOOL type_engine_scan0_class_def( Chuck_Env * env, a_Class_Def class_def );
 
-t_CKBOOL type_engine_scan1_stmt_list( Chuck_Env * env, a_Stmt_List list );
+t_CKBOOL type_engine_scan1_stmt_list( Chuck_Env * env, a_Stmt_List list, t_CKBOOL * retrieveAllControlPathsReturn = NULL );
 t_CKBOOL type_engine_scan1_stmt( Chuck_Env * env, a_Stmt stmt );
 t_CKBOOL type_engine_scan1_if( Chuck_Env * env, a_Stmt_If stmt );
 t_CKBOOL type_engine_scan1_for( Chuck_Env * env, a_Stmt_For stmt );
@@ -122,8 +122,6 @@ t_CKBOOL type_engine_scan2_cast_valid( Chuck_Env * env, t_CKTYPE to, t_CKTYPE fr
 t_CKBOOL type_engine_scan2_code_segment( Chuck_Env * env, a_Stmt_Code stmt, t_CKBOOL push = TRUE );
 t_CKBOOL type_engine_scan2_func_def( Chuck_Env * env, a_Func_Def func_def );
 t_CKBOOL type_engine_scan2_class_def( Chuck_Env * env, a_Class_Def class_def );
-t_CKBOOL type_engine_scan2_func_def( Chuck_Env * env, a_Func_Def func_def );
-t_CKBOOL type_engine_scan2_class_def( Chuck_Env * env, a_Class_Def class_def );
 
 
 
@@ -192,6 +190,7 @@ t_CKBOOL type_engine_scan0_prog( Chuck_Env * env, a_Program prog,
             break;
         }
 
+        // next section
         prog = prog->next;
     }
 
@@ -260,7 +259,7 @@ t_CKBOOL type_engine_scan0_class_def( Chuck_Env * env, a_Class_Def class_def )
     // set the fields
     the_class->xid = te_user;
     the_class->base_name = S_name(class_def->name->xid);
-    the_class->owner = env->curr;
+    the_class->owner = env->curr; CK_SAFE_ADD_REF(the_class->owner);
     the_class->array_depth = 0;
     the_class->size = sizeof(void *);
     the_class->obj_size = 0;  // TODO:
@@ -330,10 +329,10 @@ t_CKBOOL type_engine_scan0_class_def( Chuck_Env * env, a_Class_Def class_def )
         Chuck_Type * type = NULL;
 
         // allocate value
-        type = env->ckt_class->copy( env );
+        type = env->ckt_class->copy( env, env->context );
         type->actual_type = the_class;
         value = env->context->new_Chuck_Value( type, the_class->base_name );
-        value->owner = env->curr;
+        value->owner = env->curr; CK_SAFE_ADD_REF(value->owner);
         value->is_const = TRUE;
         value->is_member = FALSE;
         // add to env
@@ -359,7 +358,7 @@ t_CKBOOL type_engine_scan0_class_def( Chuck_Env * env, a_Class_Def class_def )
     }
 
     // initialize the Type info object | 1.5.0.0 (ge) added
-    initialize_object( the_class, env->ckt_class );
+    initialize_object( the_class, env->ckt_class, NULL, env->vm() );
 
 done:
 
@@ -425,6 +424,7 @@ t_CKBOOL type_engine_scan1_prog( Chuck_Env * env, a_Program prog,
             break;
         }
 
+        // next section
         prog = prog->next;
     }
 
@@ -439,10 +439,17 @@ t_CKBOOL type_engine_scan1_prog( Chuck_Env * env, a_Program prog,
 
 //-----------------------------------------------------------------------------
 // name: type_engine_scan1_stmt_list()
-// desc: ...
+// desc: check a statement list; retrieveAllControlPathsReturn is pointer
+//       to retrieve whether the statement list control paths all return
 //-----------------------------------------------------------------------------
-t_CKBOOL type_engine_scan1_stmt_list( Chuck_Env * env, a_Stmt_List list )
+t_CKBOOL type_engine_scan1_stmt_list( Chuck_Env * env, a_Stmt_List list,
+                                      t_CKBOOL * retrieveAllControlPathsReturn )
 {
+    // last non-NULL statement
+    a_Stmt lastStmt = NULL;
+    // set to NULL to begin with
+    if( retrieveAllControlPathsReturn ) *retrieveAllControlPathsReturn = FALSE;
+
     // type check the stmt_list
     while( list )
     {
@@ -450,10 +457,20 @@ t_CKBOOL type_engine_scan1_stmt_list( Chuck_Env * env, a_Stmt_List list )
         if( !type_engine_scan1_stmt( env, list->stmt ) )
             return FALSE;
 
+        // add if not NULL
+        if( retrieveAllControlPathsReturn && list->stmt != NULL )
+            lastStmt = list->stmt;
+
         // advance to the next statement
         list = list->next;
     }
 
+    // set to the last non-NULL stmt | 1.5.1.0
+    // handle cases with multiple ending ;;;
+    //     public class Class { fun void foo(){ samp => now;; } }
+    if( retrieveAllControlPathsReturn && lastStmt )
+        *retrieveAllControlPathsReturn = lastStmt->allControlPathsReturn;
+
     return TRUE;
 }
 
@@ -467,6 +484,7 @@ t_CKBOOL type_engine_scan1_stmt_list( Chuck_Env * env, a_Stmt_List list )
 t_CKBOOL type_engine_scan1_stmt( Chuck_Env * env, a_Stmt stmt )
 {
     t_CKBOOL ret = FALSE;
+    t_CKBOOL allControlPathsReturn = FALSE; // 1.5.1.0
 
     if( !stmt )
         return TRUE;
@@ -481,6 +499,23 @@ t_CKBOOL type_engine_scan1_stmt( Chuck_Env * env, a_Stmt stmt )
             ret = type_engine_scan1_if( env, &stmt->stmt_if );
             env->curr->value.pop();
             env->class_scope--;
+
+            // both control paths must end with return for this to be true | 1.5.1.0
+            if( stmt->stmt_if.if_body )
+            {
+                // check if there is ELSE-BODY
+                if( stmt->stmt_if.else_body )
+                {
+                    // both IF-BODY and ELSE-BODY should verify all control paths return
+                    allControlPathsReturn = stmt->stmt_if.if_body->allControlPathsReturn
+                                         && stmt->stmt_if.else_body->allControlPathsReturn;
+                }
+                else
+                {
+                    // if no ELSE-BODY, then no guarantee all control paths return
+                    allControlPathsReturn = FALSE;
+                }
+            }
             break;
 
         case ae_stmt_for:
@@ -529,12 +564,17 @@ t_CKBOOL type_engine_scan1_stmt( Chuck_Env * env, a_Stmt stmt )
 
         case ae_stmt_return:
             ret = type_engine_scan1_return( env, &stmt->stmt_return );
+            // this one should be easy, since this is literal return stmt | 1.5.1.0
+            allControlPathsReturn = TRUE;
             break;
 
         case ae_stmt_code:
             env->class_scope++;
             ret = type_engine_scan1_code_segment( env, &stmt->stmt_code );
             env->class_scope--;
+
+            // check if code segment (e.g., { stmt_list }) fulfills this | 1.5.1.0
+            allControlPathsReturn = stmt->stmt_code.allControlPathsReturn;
             break;
 
         case ae_stmt_break:
@@ -566,6 +606,9 @@ t_CKBOOL type_engine_scan1_stmt( Chuck_Env * env, a_Stmt stmt )
             break;
     }
 
+    // set whether all control paths associated with this stmt return | 1.5.1.0
+    stmt->allControlPathsReturn = allControlPathsReturn;
+
     return ret;
 }
 
@@ -783,14 +826,14 @@ t_CKBOOL type_engine_scan1_return( Chuck_Env * env, a_Stmt_Return stmt )
 // desc: ...
 //-----------------------------------------------------------------------------
 t_CKBOOL type_engine_scan1_code_segment( Chuck_Env * env, a_Stmt_Code stmt,
-                                        t_CKBOOL push )
+                                         t_CKBOOL push )
 {
     // class
     env->class_scope++;
     // push
     if( push ) env->curr->value.push(); // env->context->nspc.value.push();
     // do it
-    t_CKBOOL t = type_engine_scan1_stmt_list( env, stmt->stmt_list );
+    t_CKBOOL t = type_engine_scan1_stmt_list( env, stmt->stmt_list, &stmt->allControlPathsReturn );
     // pop
     if( push ) env->curr->value.pop();  // env->context->nspc.value.pop();
     // class
@@ -891,13 +934,12 @@ t_CKBOOL type_engine_scan1_exp_binary( Chuck_Env * env, a_Exp_Binary binary )
 {
     a_Exp cl = binary->lhs, cr = binary->rhs;
 
-    // type check the lhs and rhs
+    // scan lhs
     t_CKBOOL left = type_engine_scan1_exp( env, cl );
+    if( !left ) return FALSE;
+    // scan rhs
     t_CKBOOL right = type_engine_scan1_exp( env, cr);
-
-    // if either fails, then return FALSE
-    if( !left || !right )
-        return FALSE;
+    if( !right ) return FALSE;
 
     // cross chuck
     while( cr )
@@ -1348,11 +1390,6 @@ t_CKBOOL type_engine_scan1_func_def( Chuck_Env * env, a_Func_Def f )
 {
     a_Arg_List arg_list = NULL;
     t_CKUINT count = 0;
-    // t_CKBOOL has_code = FALSE;
-    // Chuck_Value * v = NULL;
-
-    // if not imported, then check to make sure no reserved word conflict
-    // if( f->s_type != ae_func_builtin )  // TODO: fix this
 
     // check if reserved
     if( type_engine_check_reserved( env, f->name, f->where ) )
@@ -1420,7 +1457,6 @@ t_CKBOOL type_engine_scan1_func_def( Chuck_Env * env, a_Func_Def f )
         if( !arg_list->type )
         {
             // TODO: try to resolve
-            // EM_error2( arg_list->where, "in function '%s':", S_name(f->name) );
             EM_error2( arg_list->where,
                 "...in argument %i '%s' of function '%s(.)'",
                 count, S_name(arg_list->var_decl->xid), S_name(f->name) );
@@ -1441,6 +1477,9 @@ t_CKBOOL type_engine_scan1_func_def( Chuck_Env * env, a_Func_Def f )
         goto error;
     }
 
+    // propagate from stmt_code to the outer stmt | 1.5.1.0
+    if( f->code ) f->code->allControlPathsReturn = f->code->stmt_code.allControlPathsReturn;
+
     return TRUE;
 
 error:
@@ -1504,6 +1543,7 @@ t_CKBOOL type_engine_scan2_prog( Chuck_Env * env, a_Program prog,
             break;
         }
 
+        // next section
         prog = prog->next;
     }
 
@@ -1970,13 +2010,12 @@ t_CKBOOL type_engine_scan2_exp_binary( Chuck_Env * env, a_Exp_Binary binary )
 {
     a_Exp cl = binary->lhs, cr = binary->rhs;
 
-    // type check the lhs and rhs
+    // scan lhs
     t_CKBOOL left = type_engine_scan2_exp( env, cl );
+    if( !left ) return FALSE;
+    // scan rhs
     t_CKBOOL right = type_engine_scan2_exp( env, cr);
-
-    // if either fails, then return FALSE
-    if( !left || !right )
-        return FALSE;
+    if( !right ) return FALSE;
 
     // cross chuck
     while( cr )
@@ -2208,6 +2247,192 @@ t_CKBOOL type_engine_scan2_array_subscripts( Chuck_Env * env, a_Exp exp_list )
 
 
 
+//-----------------------------------------------------------------------------
+// name: type_engine_scan2_exp_decl_create() | 1.5.1.1 (ge)
+//       *** adapted from type_engine_scan2_exp_decl() pre-1.5.0.8
+//       *** which became type_engine_check_exp_decl_part1() in 1.5.0.8
+// reason: 'auto' needs more context before it can processed | 1.5.0.8 (ge)
+//       however, class variables are meant to be accessible out-of-order
+//       and needs decl created in an earlier pass | 1.5.1.1 (ge)
+// desc: this can now be called either from type_scan if not 'auto' in order
+//       to support accessing class member decls from anywhere (e.g., in the
+//       same file, above the actual decl); or it can be deferred in the case
+//       of 'auto' from chuck_type | 1.5.1.1 (ge)
+// caveat: 'auto' is disallowed for now as a top-level class member decl type
+//-----------------------------------------------------------------------------
+t_CKBOOL type_engine_scan2_exp_decl_create( Chuck_Env * env, a_Exp_Decl decl )
+{
+    a_Var_Decl_List list = decl->var_decl_list;
+    a_Var_Decl var_decl = NULL;
+    Chuck_Type * type = NULL;
+    Chuck_Value * value = NULL;
+    t_CKBOOL do_alloc = TRUE;
+    t_CKBOOL is_first_in_list = TRUE;
+
+    // check to see if Part 1 was already processed | 1.5.1.0 (ge)
+    // NOTE Part 1 (migrated here from scan2b_exp_decl() in 1.5.0.8 to support
+    // 'auto', can only be run once as it creates and adds values into the
+    // current scope; yet a decl expression could be checked more than once
+    // due to chained chuck statements like: `440 => float f => osc.freq;`
+    if( list && list->var_decl && list->var_decl->value )
+        return TRUE;
+
+    // retrieve the type
+    type = decl->ck_type;
+    // make sure it's not NULL
+    assert( type != NULL );
+
+    // check to see type is not void or auto
+    // by this point, any 'auto' should have been resolved
+    if( type->size == 0 )
+    {
+        // see if auto was declared
+        if( isa( type, env->ckt_auto ) )
+        {
+            EM_error2( decl->type->where,
+                "cannot use 'auto' type variable declaration here..." );
+            EM_error2( 0, "(hint: 'auto' requires either initialization or specific contexts)" );
+        }
+        else
+        {
+            EM_error2( decl->where,
+                "cannot declare variables of size '0' (i.e. 'void')..." );
+        }
+        return FALSE;
+    }
+
+    // T @ foo?
+    do_alloc = !decl->type->ref;
+
+    // make sure complete
+    if( /*!t->is_complete &&*/ do_alloc )
+    {
+        // check to see if class inside itself
+        if( env->class_def && equals( type, env->class_def ) && env->class_scope == 0 )
+        {
+            EM_error2( decl->where,
+                "...(note: object of type '%s' declared inside itself)",
+                type->c_name() );
+            return FALSE;
+        }
+    }
+
+    // primitive
+    if( (isprim( env, type ) || isa( type, env->ckt_string )) && decl->type->ref )  // TODO: string
+    {
+        EM_error2( decl->where,
+            "cannot declare references (@) of primitive type '%s'...",
+            type->c_name() );
+        EM_error2( decl->where,
+            "...(primitive types: 'int', 'float', 'time', 'dur')" );
+        return FALSE;
+    }
+
+    // loop through the variables
+    while( list )
+    {
+        // 1.4.2.0 (ge) | reset the type variable to the lead type in the decl
+        // e.g., for cases like int x[2], y;
+        // ...this is so y would not be associated with x's array type
+        type = decl->ck_type;
+
+        // get the decl
+        var_decl = list->var_decl;
+        // 1.4.2.0 (ge) | by default, copy the decl type reference bit
+        // this could be overwritten later as appropriate, e.g., by array vars
+        var_decl->ref = decl->type->ref;
+
+        // check if reserved
+        if( type_engine_check_reserved( env, var_decl->xid, var_decl->where ) )
+        {
+            EM_error2( var_decl->where,
+                "...in variable declaration", S_name(var_decl->xid) );
+            return FALSE;
+        }
+
+        // check if locally defined
+        if( env->curr->lookup_value( var_decl->xid, FALSE ) )
+        {
+            EM_error2( var_decl->where,
+                "'%s' has already been defined in the same scope",
+                S_name(var_decl->xid) );
+            return FALSE;
+        }
+
+        // check if array
+        if( var_decl->array != NULL )
+        {
+            // 1.4.2.0 (ge) was: decl->type->ref;
+            var_decl->ref = ( var_decl->array->exp_list == NULL );
+            // the declaration type | 1.4.2.0 (ge) fixed for multiple decl (e.g., int x[1], y[2];)
+            Chuck_Type * t2 = decl->ck_type; // was: type, which won't work if more than one var declared
+
+            // create the new array type
+            type = new_array_type(
+                env,  // the env
+                env->ckt_array,  // the array base class
+                var_decl->array->depth,  // the depth of the new type
+                t2,  // the 'array_type'
+                env->curr  // the owner namespace
+            );
+
+            // 1.4.2.0 (ge) | assign new array type to current var decl
+            // for handling the following kind of multi-var declarations
+            //   int x[1], y[2];
+            //   int x, y[1];
+            // set reference : var_decl->ck_type = type;
+            CK_SAFE_REF_ASSIGN( var_decl->ck_type, type );
+
+            // 1.4.2.0 (ge) | if one and only one variable, then update decl->ck_type
+            // otherwise, the variables could have different array depths, and therefore different types
+            // also note: cannot => to a multi-variable declaration (e.g., 5 => int x, y;)
+            // this is to support array initialization (e.g., [ [1,2], [3,4] ] @=> int x[][];)
+            if( is_first_in_list && list->next == NULL )
+            {
+                // set reference : var_decl->ck_type = type;
+                CK_SAFE_REF_ASSIGN( decl->ck_type, type );
+            }
+        }
+
+        // enter into value binding
+        env->curr->value.add( var_decl->xid,
+            value = env->context->new_Chuck_Value( type, S_name(var_decl->xid) ) );
+
+        // remember the owner
+        value->owner = env->curr; CK_SAFE_ADD_REF( value->owner );
+        value->owner_class = env->func ? NULL : env->class_def;
+        value->is_member = ( env->class_def != NULL &&
+                             env->class_scope == 0 &&
+                             env->func == NULL && !decl->is_static );
+        value->is_context_global = ( env->class_def == NULL && env->func == NULL );
+        value->addr = var_decl->addr;
+        // flag it until the decl is checked
+        value->is_decl_checked = FALSE;
+        // flag as global
+        value->is_global = decl->is_global;
+        // flag as const | 1.5.1.3 (ge) added
+        value->is_const = decl->is_const;
+
+        // dependency tracking: remember the code position of the DECL | 1.5.0.8
+        // do only if file-top-level or class-top-level, but not global
+        if( (value->is_member || value->is_context_global) && !value->is_global )
+            value->depend_init_where = var_decl->where;
+
+        // remember the value
+        var_decl->value = value;
+
+        // the next var decl
+        list = list->next;
+        // 1.4.2.0 (ge) | added
+        is_first_in_list = FALSE;
+    }
+
+    return TRUE;
+}
+
+
+
+
 //-----------------------------------------------------------------------------
 // name: type_engine_scan2_exp_decl( )
 // desc: ...
@@ -2245,6 +2470,27 @@ t_CKBOOL type_engine_scan2_exp_decl( Chuck_Env * env, a_Exp_Decl decl )
         list = list->next;
     }
 
+    // check whether decl type is auto | 1.5.1.1
+    if( !isa(decl->ck_type, env->ckt_auto) )
+    {
+        // not auto? go ahead and create the decl
+        if( !type_engine_scan2_exp_decl_create( env, decl ) )
+            return FALSE;
+    }
+    else // decl type is auto
+    {
+        // disallow 'auto' for top-level class member variables
+        if( env->class_def && env->class_scope == 0 )
+        {
+            EM_error2( decl->type->where,
+                "cannot use 'auto' type for top-level class variables..." );
+            return FALSE;
+        }
+
+        // nothing more to do for now
+        // defer decl creation until chuck_type has a chance to infer the auto type
+    }
+
     return TRUE;
 }
 
@@ -2394,6 +2640,56 @@ t_CKBOOL type_engine_scan2_class_def( Chuck_Env * env, a_Class_Def class_def )
     env->curr = env->nspc_stack.back();
     env->nspc_stack.pop_back();
 
+    // the parent class | 1.5.1.5 (ge) moved from chuck_type module for earlier info
+    t_CKTYPE t_parent = NULL;
+
+    // make sure inheritance
+    // TODO: sort!
+    if( class_def->ext )
+    {
+        // if extend
+        if( class_def->ext->extend_id )
+        {
+            // find the type
+            t_parent = type_engine_find_type( env, class_def->ext->extend_id );
+            if( !t_parent )
+            {
+                EM_error2( class_def->ext->extend_id->where,
+                    "undefined super class '%s' in definition of class '%s'",
+                    type_path(class_def->ext->extend_id), S_name(class_def->name->xid) );
+                return FALSE;
+            }
+
+            // must not be primitive
+            if( isprim( env, t_parent ) )
+            {
+                EM_error2( class_def->ext->extend_id->where,
+                    "cannot extend primitive type '%s'",
+                    t_parent->c_name() );
+                EM_error2( 0, "...(primitive types: 'int', 'float', 'time', 'dur', etc.)" );
+                return FALSE;
+            }
+        }
+
+        // TODO: interface
+    }
+
+    // by default object
+    if( !t_parent ) t_parent = env->ckt_object;
+
+    // check for fun
+    assert( env->context != NULL );
+    assert( class_def->type != NULL );
+    assert( class_def->type->info != NULL );
+
+    // retrieve the new type (created in scan_class_def)
+    the_class = class_def->type;
+
+    // set fields not set in scan
+    the_class->parent = t_parent; CK_SAFE_ADD_REF(t_parent);
+    // inherit ugen_info data from parent PLD
+    the_class->ugen_info = t_parent->ugen_info; CK_SAFE_ADD_REF(t_parent->ugen_info);
+
     return ret;
 }
 
@@ -2424,6 +2720,7 @@ t_CKBOOL type_engine_scan2_func_def( Chuck_Env * env, a_Func_Def f )
     t_CKUINT count = 0;
     Chuck_Func * overfunc = NULL;
     // t_CKBOOL has_code = FALSE;  // use this for both user and imported
+    t_CKBOOL op_overload = ( f->op2overload != ae_op_none );
 
     // see if we are already in a function definition
     if( env->func != NULL )
@@ -2465,7 +2762,7 @@ t_CKBOOL type_engine_scan2_func_def( Chuck_Env * env, a_Func_Def f )
     if( overload )
     {
         // make the new name
-        func_name += "@" + itoa( ++overload->func_num_overloads ) + "@" + env->curr->name;
+        func_name += "@" + ck_itoa( ++overload->func_num_overloads ) + "@" + env->curr->name;
     }
     else
     {
@@ -2494,7 +2791,7 @@ t_CKBOOL type_engine_scan2_func_def( Chuck_Env * env, a_Func_Def f )
     if( f->s_type == ae_func_builtin )
     {
         // we can emit code now
-        func->code = new Chuck_VM_Code;
+        func->code = new Chuck_VM_Code; func->code->add_ref();
         // whether the function needs 'this'
         func->code->need_this = func->is_member;
         // is static inside
@@ -2507,7 +2804,7 @@ t_CKBOOL type_engine_scan2_func_def( Chuck_Env * env, a_Func_Def f )
     type = env->context->new_Chuck_Type( env );
     type->xid = te_function;
     type->base_name = "[function]";
-    type->parent = env->ckt_function;
+    type->parent = env->ckt_function; // TODO: reference count the parent
     type->size = sizeof(void *);
     type->func = func;
 
@@ -2516,15 +2813,15 @@ t_CKBOOL type_engine_scan2_func_def( Chuck_Env * env, a_Func_Def f )
     // it is const
     value->is_const = TRUE;
     // remember the owner
-    value->owner = env->curr;
-    value->owner_class = env->class_def;
+    value->owner = env->curr; CK_SAFE_ADD_REF( value->owner );
+    value->owner_class = env->class_def; CK_SAFE_ADD_REF( value->owner_class );
     value->is_member = func->is_member;
-       // is global context
+    // is global context
     value->is_context_global = env->class_def == NULL;
     // remember the func
-    value->func_ref = func; func->add_ref(); // add reference TODO: break cycle?
+    value->func_ref = func; CK_SAFE_ADD_REF( value->func_ref ); // add reference TODO: break cycle?
     // remember the value
-    func->value_ref = value; value->add_ref(); // add reference TODO: break cycle?
+    func->value_ref = value; CK_SAFE_ADD_REF( func->value_ref ); // add reference TODO: break cycle?
 
     // set the func
     f->ck_func = func; func->add_ref(); // add reference
@@ -2585,7 +2882,7 @@ t_CKBOOL type_engine_scan2_func_def( Chuck_Env * env, a_Func_Def f )
         // check if reserved
         if( type_engine_check_reserved( env, arg_list->var_decl->xid, arg_list->where ) )
         {
-            EM_error2( arg_list->where, "in function '%s'", S_name(f->name) );
+            EM_error2( arg_list->where, "in function '%s'", func->signature().c_str() ); // S_name(f->name) );
             goto error;
         }
 
@@ -2622,10 +2919,10 @@ t_CKBOOL type_engine_scan2_func_def( Chuck_Env * env, a_Func_Def f )
             // should be partial and empty []
             if( arg_list->var_decl->array->exp_list )
             {
-                EM_error2( arg_list->where, "in function '%s':", S_name(f->name) );
+                EM_error2( arg_list->where, "in function '%s':", func->signature().c_str() ); // S_name(f->name) );
                 EM_error2( arg_list->where, "argument %i '%s' must be defined with empty []'s",
                     count, S_name(arg_list->var_decl->xid) );
-                return FALSE;
+                goto error; // return FALSE;
             }
 
             // create the new array type
@@ -2647,10 +2944,11 @@ t_CKBOOL type_engine_scan2_func_def( Chuck_Env * env, a_Func_Def f )
         v = env->context->new_Chuck_Value(
             arg_list->type, S_name(arg_list->var_decl->xid) );
         // remember the owner
-        v->owner = env->curr;
+        v->owner = env->curr; CK_SAFE_ADD_REF( v->owner );
         // function args not owned
-        v->owner_class = NULL;
+        v->owner_class = NULL; CK_SAFE_ADD_REF( v->owner_class );
         v->is_member = FALSE;
+        v->is_const = FALSE; // 1.5.1.3 (ge) added explicitly for future reference
         // later: add as value
         // symbols.push_back( arg_list );
         // values.push_back( v );
@@ -2714,6 +3012,22 @@ t_CKBOOL type_engine_scan2_func_def( Chuck_Env * env, a_Func_Def f )
                 // check
                 if( same_arg_lists(lhs, rhs) )
                 {
+                    ae_Operator op = f->op2overload;
+                    // check if this is an overloading of both post and pre (e.g., -- and ++)
+                    if( op_overload && env->op_registry.unaryPreOverloadable(op) &&
+                                       env->op_registry.unaryPostOverloadable(op) )
+                    {
+                        // check what kind of unary overload
+                        if( (f->overload_post && env->op_registry.lookup_overload(lhs->type,op) == NULL )
+                           || (!f->overload_post && env->op_registry.lookup_overload(op,lhs->type) ) )
+                        {
+                            // next overfunc
+                            overfunc = overfunc->next;
+                            // keep going, this is ok
+                            continue;
+                        }
+                    }
+
                     EM_error2( f->where, "cannot overload functions with identical arguments..." );
                     if( env->class_def )
                     {
@@ -2734,6 +3048,14 @@ t_CKBOOL type_engine_scan2_func_def( Chuck_Env * env, a_Func_Def f )
         }
     }
 
+    // operator overload | 1.5.1.5 (ge) added
+    if( op_overload )
+    {
+        // scan operator overload
+        if( !type_engine_scan_func_op_overload( env, f ) )
+            return FALSE;
+    }
+
     // add as value
     env->curr->value.add( value->name, value );
     // enter the name into the function table
@@ -2755,7 +3077,7 @@ t_CKBOOL type_engine_scan2_func_def( Chuck_Env * env, a_Func_Def f )
     assert( f->code == NULL || f->code->s_type == ae_stmt_code );
     if( f->code && !type_engine_scan2_code_segment( env, &f->code->stmt_code, FALSE ) )
     {
-        EM_error2( 0, "...in function '%s'", S_name(f->name) );
+        EM_error2( 0, "...in function '%s'", func->signature().c_str() ); // S_name(f->name) );
         goto error;
     }
 
@@ -2771,8 +3093,13 @@ error:
     // clean up
     if( func )
     {
+        // reset
         env->func = NULL;
-        CK_SAFE_RELEASE(func);
+        // break func_ref <-> value_ref cycle, at least here
+        CK_SAFE_RELEASE( func->value_ref );
+        // value, type, func shouldn't be released here | 1.5.1.3
+        // should be cleanup with the overall context
+        // CK_SAFE_RELEASE(value); CK_SAFE_RELEASE(type); CK_SAFE_RELEASE(func);
     }
 
     return FALSE;
