@@ -53,8 +53,8 @@
 //-----------------------------------------------------------------------------
 // vm defines
 //-----------------------------------------------------------------------------
-#define CVM_MEM_STACK_SIZE          (0x1 << 16)
-#define CVM_REG_STACK_SIZE          (0x1 << 14)
+#define CKVM_MEM_STACK_SIZE          (0x1 << 16)
+#define CKVM_REG_STACK_SIZE          (0x1 << 14)
 
 
 // forward references
@@ -64,6 +64,7 @@ struct Chuck_VM_Func;
 struct Chuck_VM_FTable;
 struct Chuck_Msg;
 struct Chuck_Globals_Manager; // added 1.4.1.0 (jack)
+struct Chuck_Instr_Reg_Push_Imm; // 1.5.1.5 (ge)
 class CBufferSimple;
 #ifndef __DISABLE_SERIAL__
 // hack: spencer?
@@ -89,7 +90,7 @@ public:
     ~Chuck_VM_Stack();
 
 public:
-    // initialize stack
+    // initialize stack of at least 'size' bytes
     t_CKBOOL initialize( t_CKUINT size );
     // shutdown and cleanup stack
     t_CKBOOL shutdown();
@@ -108,6 +109,7 @@ public: // linked list
 
 public: // state
     t_CKBOOL m_is_init;
+    t_CKUINT m_size; // 1.5.1.5
 };
 
 
@@ -133,7 +135,7 @@ public:
 
     // name of this code
     std::string name;
-    // the depth of any function arguments
+    // the depth of any function arguments (in bytes)
     t_CKUINT stack_depth;
     // whether the function needs 'this' pointer or not
     t_CKBOOL need_this;
@@ -171,8 +173,8 @@ public:
 
     // initialize shred
     t_CKBOOL initialize( Chuck_VM_Code * c,
-                         t_CKUINT mem_st_size = CVM_MEM_STACK_SIZE,
-                         t_CKUINT reg_st_size = CVM_REG_STACK_SIZE );
+                         t_CKUINT mem_st_size = 0,
+                         t_CKUINT reg_st_size = 0 );
     // shutdown shred
     t_CKBOOL shutdown();
     // run the shred on vm
@@ -184,12 +186,22 @@ public:
     t_CKBOOL add( Chuck_UGen * ugen );
     // unassociate ugen with shred
     t_CKBOOL remove( Chuck_UGen * ugen );
+    // detach all associate ugens | 1.5.1.5 (ge) added
+    void detach_ugens();
 
     // add parent object reference (added 1.3.1.2)
     t_CKVOID add_parent_ref( Chuck_Object * obj );
     // add get shred id | 1.5.0.8 (ge)
     t_CKUINT get_id() const { return this->xid; }
 
+    // affects children shreds sporked from this one
+    // mem is memory / call stack (for local vars)
+    t_CKINT childSetMemSize( t_CKINT sizeInBytes );
+    t_CKINT childGetMemSize( );
+    // reg is rester / operand stack (for evaluating expressions)
+    t_CKINT childSetRegSize( t_CKINT sizeInBytes );
+    t_CKINT childGetRegSize( );
+
     #ifndef __DISABLE_SERIAL__
     // HACK - spencer (added 1.3.2.0)
     // add/remove SerialIO devices to close on shred exit
@@ -202,23 +214,34 @@ public:
 // data
 //-----------------------------------------------------------------------------
 public: // machine components
-    // stacks
-    Chuck_VM_Stack * mem;
-    Chuck_VM_Stack * reg;
+    // stacks; mem for "memory" and reg for "register"
+    Chuck_VM_Stack * mem; // call stack
+    Chuck_VM_Stack * reg; // operand stack
 
     // ref to base stack - if this is the root, then base is mem
     Chuck_VM_Stack * base_ref;
 
-    // code
+    // current vm code being run (this could change, e.g., across func calls)
     Chuck_VM_Code * code;
-    Chuck_VM_Code * code_orig; // the one to release
+    // the original vm code attached to this shred (also: the one to release)
+    Chuck_VM_Code * code_orig;
+    // instructions
     Chuck_Instr ** instr;
-    Chuck_VM_Shred * parent;
-    std::map<t_CKUINT, Chuck_VM_Shred *> children;
-    t_CKUINT pc;
 
+    // parent shred
+    Chuck_VM_Shred * parent;
+    // children shreds
+    std::map<t_CKUINT, Chuck_VM_Shred *> children;
+
+    // child stack size hints | 1.5.1.5
+    t_CKINT memStackSize;
+    t_CKINT regStackSize;
+
+    // program counter
+    t_CKUINT pc;
     // time
     t_CKTIME now;
+    // when started, in chuck time, relative to start of VM
     t_CKTIME start;
     // vm reference
     Chuck_VM * vm_ref;
@@ -263,6 +286,30 @@ public: // ge: 1.3.5.3
     // loop counter pointer stack
     std::vector<t_CKUINT *> m_loopCounters;
 
+public: // immediate mode temporal restriction | 1.5.1.5 (ge)
+    // while in this mode, exception will be thrown on any time ops:
+    // 1) if shred advances time (even by 0 duration)
+    // 2) if shred waits on event
+    // 3) if shred yields, or calls Machine.eval()
+    //    which implicits yields to runs code on a new shred
+    // this is typically set in specific cases where the programmer
+    // is expected to provide a callback function that must return
+    // immediately without any shreduling; for example Chugen.tick( float in )
+    // or GGen.update( float dt );
+
+    // toggle immediate mode
+    void setImmediateMode( t_CKBOOL onOff ) { is_immediate_mode = onOff; }
+    // check if shred is in immediate mode
+    t_CKBOOL immediateMode() const { return is_immediate_mode; }
+    // check if shred is in immediate mode
+    t_CKBOOL immediateModeVioation() const { return is_immediate_mode_violation; }
+    // test and report for immediate mode violations
+    t_CKBOOL checkImmediatModeException( t_CKUINT linepos = 0 );
+
+protected:
+    t_CKBOOL is_immediate_mode;
+    t_CKBOOL is_immediate_mode_violation;
+
 #ifndef __DISABLE_SERIAL__
 private:
     // serial IO list for event synchronization
@@ -273,6 +320,30 @@ private:
 
 
 
+//-----------------------------------------------------------------------------
+// name: struct Chuck_VM_Shreds_Watcher
+// desc: ChucK virtual machine
+//-----------------------------------------------------------------------------
+struct Chuck_VM_Shreds_Watcher
+{
+    // function pointer to call
+    f_shreds_watcher cb;
+    // user data
+    void * userdata;
+
+    // constructor
+    Chuck_VM_Shreds_Watcher( f_shreds_watcher f = NULL, void * data = NULL ) : cb(f), userdata(data) { }
+    // copy constructor
+    Chuck_VM_Shreds_Watcher( const Chuck_VM_Shreds_Watcher & other )
+    : cb(other.cb), userdata(other.userdata) { }
+    // ==
+    bool operator ==( const Chuck_VM_Shreds_Watcher & other )
+    { return this->cb == other.cb; }
+};
+
+
+
+
 //-----------------------------------------------------------------------------
 // name: struct Chuck_VM_Shred_Status
 // desc: status pertaining to a single shred
@@ -333,7 +404,7 @@ public:
 // name: struct Chuck_VM_Shreduler
 // desc: a ChucK shreduler shredules shreds
 //-----------------------------------------------------------------------------
-struct Chuck_VM_Shreduler : Chuck_Object
+struct Chuck_VM_Shreduler : public Chuck_Object
 {
 //-----------------------------------------------------------------------------
 // functions
@@ -460,7 +531,7 @@ public:
 // name: struct Chuck_VM
 // desc: ChucK virtual machine
 //-----------------------------------------------------------------------------
-struct Chuck_VM : Chuck_Object
+struct Chuck_VM : public Chuck_Object
 {
 //-----------------------------------------------------------------------------
 // functions
@@ -502,10 +573,10 @@ public: // shredsuck
     // get reference to shreduler
     Chuck_VM_Shreduler * shreduler() const;
     // the next spork ID
-    t_CKUINT next_id();
+    t_CKUINT next_id( const Chuck_VM_Shred * shred = NULL );
     // the last used spork ID
     t_CKUINT last_id() const;
-    // reset ID to lowest current ID + 1; returns what next ID would be
+    // reset ID to highest current ID + 1; returns what next ID would be
     t_CKUINT reset_id();
     // the current chuck time | 1.5.0.8
     t_CKTIME now() const;
@@ -536,15 +607,18 @@ public: // VM message queue
     // process a VM message immediately (synchronous but not thread-safe)
     // NOTE assumes msg is dynamically allocated using `new`; will be deleted by VM
     // NOTE this processes the msg immediately on calling thread
-    t_CKUINT process_msg( Chuck_Msg * msg );
+    t_CKUINT process_msg( Chuck_Msg * & msg );
     // get reply from reply buffer
     Chuck_Msg * get_reply();
 
     // CBufferSimple added 1.3.0.0 to fix uber-crash
     t_CKBOOL queue_event( Chuck_Event * event, t_CKINT num_msg = 1, CBufferSimple * buffer = NULL );
-    // added 1.3.0.0 to fix uber-crash
+    // added 1.3.0.0 to fix uber-crash (allocates and attaches new buffer)
     CBufferSimple * create_event_buffer();
+    // added 1.3.0.0 to fix uber-crash (detaches and deletes buffer)
     void destroy_event_buffer( CBufferSimple * buffer );
+    // added 1.5.1.3 for granularity in handling synchronization
+    void detach_event_buffer_without_delete( CBufferSimple * buffer );
 
 public: // get error
     const char * last_error() const
@@ -559,6 +633,15 @@ public:
     // 1.4.1.0 (jack): get associated globals manager
     Chuck_Globals_Manager * globals_manager() const { return m_globals_manager; }
 
+public:
+    // subscribe shreds watcher callback | 1.5.1.5
+    void subscribe_watcher( f_shreds_watcher cb, t_CKUINT options, void * data = NULL );
+    // notify watchers | 1.5.1.5
+    void notify_watchers( ckvmShredsWatcherFlag which, Chuck_VM_Shred * shred,
+                          std::list<Chuck_VM_Shreds_Watcher> & v );
+    // remove shreds watcher callback | 1.5.1.5
+    void remove_watcher( f_shreds_watcher cb );
+
 //-----------------------------------------------------------------------------
 // data
 //-----------------------------------------------------------------------------
@@ -598,8 +681,8 @@ protected:
     // remove all shreds from VM
     void removeAll();
     // free shred
-    t_CKBOOL free( Chuck_VM_Shred * shred, t_CKBOOL cascade,
-                   t_CKBOOL dec = TRUE );
+    t_CKBOOL free_shred( Chuck_VM_Shred * shred, t_CKBOOL cascade,
+                         t_CKBOOL dec = TRUE );
     void dump_shred( Chuck_VM_Shred * shred );
     void release_dump();
 
@@ -611,6 +694,7 @@ protected:
     Chuck_VM_Shred * m_shreds;
     t_CKUINT m_num_shreds;
     t_CKUINT m_shred_id;
+    t_CKBOOL m_shred_check4dupes; // 1.5.1.5
     Chuck_VM_Shreduler * m_shreduler;
     // place to put dumped shreds
     std::vector<Chuck_VM_Shred *> m_shred_dump;
@@ -627,6 +711,13 @@ protected:
 protected:
     // 1.4.1.0 (jack): manager for global variables
     Chuck_Globals_Manager * m_globals_manager;
+
+protected:
+    // 1.5.1.5 (ge & andrew) shreds watchers
+    std::list<Chuck_VM_Shreds_Watcher> m_shreds_watchers_spork;
+    std::list<Chuck_VM_Shreds_Watcher> m_shreds_watchers_remove;
+    std::list<Chuck_VM_Shreds_Watcher> m_shreds_watchers_suspend;
+    std::list<Chuck_VM_Shreds_Watcher> m_shreds_watchers_activate;
 };
 
 
@@ -674,10 +765,14 @@ struct Chuck_Msg
     Chuck_VM_Code * code;
     // VM shred pointer, as applicable
     Chuck_VM_Shred * shred;
+    // parent shred, as applicable
+    Chuck_VM_Shred * parent;
     // time, as applicable
     t_CKTIME when;
     // pointer to status struct, as applicable
     Chuck_VM_Status * status;
+    // whether to always add | 1.5.1.5
+    t_CKBOOL alwaysAdd;
 
     // reply callback
     ck_msg_func reply_cb;
@@ -726,6 +821,43 @@ struct Chuck_Msg
 
 
 
+//-----------------------------------------------------------------------------
+// name: struct Chuck_VM_MFunInvoker | 1.5.1.5 (ge)
+// desc: construct for calling chuck-defined member functions from c++,
+//       either from VM execution or outside the VM execution context
+//-----------------------------------------------------------------------------
+struct Chuck_VM_MFunInvoker
+{
+public:
+    // constructor
+    Chuck_VM_MFunInvoker();
+    // destructor
+    ~Chuck_VM_MFunInvoker();
+
+public:
+    // set up the invoker; needed before invoke()
+    t_CKBOOL setup( Chuck_Func * func, t_CKUINT func_vt_offset,
+                    Chuck_VM * vm, Chuck_VM_Shred * caller );
+    // invoke the member function
+    Chuck_DL_Return invoke( Chuck_Object * obj,
+                            const std::vector<Chuck_DL_Arg> & args );
+    // clean up
+    void cleanup();
+
+public:
+    // dedicated shred to call the mfun on
+    Chuck_VM_Shred * shred;
+    // instructions for args (to be filled on invoke)
+    std::vector<Chuck_Instr *> instr_args;
+    // instruction to update on invoke: pushing this pointer
+    Chuck_Instr_Reg_Push_Imm * instr_pushThis;
+    // instruction to update on invoke: pushing the var to receive return
+    Chuck_Instr_Reg_Push_Imm * instr_pushReturnVar;
+};
+
+
+
+
 //-----------------------------------------------------------------------------
 // VM debug macros | 1.5.0.5 (ge) added
 // these are governed by the presence of __CHUCK_DEBUG__ (e.g., from makefile)
