@@ -447,6 +447,7 @@ DLL_QUERY libmath_query( Chuck_DL_Query * QUERY )
     //! see \example math.ck
     QUERY->add_svar( QUERY, "float", "PI", TRUE, &g_pi );
     QUERY->doc_var( QUERY, "An approximation of pi. (Same as global keyword 'pi'.)" );
+
     // 1.4.1.0 (ge): special disable to avoid conflict (don't forget to reenabled)
     type_engine_enable_reserved( env, "pi", FALSE );
     QUERY->add_svar( QUERY, "float", "pi", TRUE, &g_pi );
@@ -982,8 +983,8 @@ CK_DLL_SFUN( gauss_impl )
 // cossim (ge) | added 1.5.0.0
 CK_DLL_SFUN( cossim_impl )
 {
-    Chuck_Array8 * a = (Chuck_Array8 *)GET_NEXT_OBJECT( ARGS );
-    Chuck_Array8 * b = (Chuck_Array8 *)GET_NEXT_OBJECT( ARGS );
+    Chuck_ArrayFloat * a = (Chuck_ArrayFloat *)GET_NEXT_OBJECT( ARGS );
+    Chuck_ArrayFloat * b = (Chuck_ArrayFloat *)GET_NEXT_OBJECT( ARGS );
     t_CKINT size = 0;
 
     // in case of error
@@ -1110,8 +1111,8 @@ CK_DLL_SFUN( cossim4d_impl )
 // euclidean (ge) | added 1.5.0.0
 CK_DLL_SFUN( euclidean_impl )
 {
-    Chuck_Array8 * a = (Chuck_Array8 *)GET_NEXT_OBJECT( ARGS );
-    Chuck_Array8 * b = (Chuck_Array8 *)GET_NEXT_OBJECT( ARGS );
+    Chuck_ArrayFloat * a = (Chuck_ArrayFloat *)GET_NEXT_OBJECT( ARGS );
+    Chuck_ArrayFloat * b = (Chuck_ArrayFloat *)GET_NEXT_OBJECT( ARGS );
     t_CKINT size = 0;
 
     // in case of error
