@@ -259,9 +259,14 @@ DLL_QUERY libstd_query( Chuck_DL_Query * QUERY )
 
     // add getenv
     QUERY->add_sfun( QUERY, getenv_impl, "string", "getenv" ); //! fetch environment variable
-    QUERY->add_arg( QUERY, "string", "value" );
+    QUERY->add_arg( QUERY, "string", "key" );
     QUERY->doc_func( QUERY, "get the value of an environment variable (e.g., PATH)." );
 
+    QUERY->add_sfun( QUERY, getenv2_impl, "string", "getenv" );//! fetch environment variable with default
+    QUERY->add_arg( QUERY, "string", "key" );
+    QUERY->add_arg( QUERY, "string", "default" );
+    QUERY->doc_func( QUERY, "get the value of an environment variable, returning the provided default if unset." );
+
     // add setenv
     QUERY->add_sfun( QUERY, setenv_impl, "int", "setenv" ); //! set environment variable
     QUERY->add_arg( QUERY, "string", "key" );
@@ -332,6 +337,22 @@ DLL_QUERY libstd_query( Chuck_DL_Query * QUERY )
     QUERY->add_arg( QUERY, "float", "dstmax" );
     QUERY->doc_func( QUERY, "Scale a float from source range to destination range." );
 
+    // add range functions | 1.5.1.1 (nshaheed)
+    QUERY->add_sfun(QUERY, range1_impl, "int[]", "range"); //! fetch environment variable
+    QUERY->add_arg(QUERY, "int", "stop");
+    QUERY->doc_func(QUERY, "Return array containing the range [0,stop).");
+
+    QUERY->add_sfun(QUERY, range2_impl, "int[]", "range"); //! fetch environment variable
+    QUERY->add_arg(QUERY, "int", "start");
+    QUERY->add_arg(QUERY, "int", "stop");
+    QUERY->doc_func(QUERY, "Return array containing the range [start,stop).");
+
+    QUERY->add_sfun(QUERY, range3_impl, "int[]", "range"); //! fetch environment variable
+    QUERY->add_arg(QUERY, "int", "start");
+    QUERY->add_arg(QUERY, "int", "stop");
+    QUERY->add_arg(QUERY, "int", "step");
+    QUERY->doc_func(QUERY, "Return array containing values from start up to (but not including) stop, hopping by step.");
+
     // finish class
     QUERY->end_class( QUERY );
 
@@ -774,7 +795,7 @@ CK_DLL_SFUN( itoa_impl )
     t_CKINT i = GET_CK_INT(ARGS);
     // TODO: memory leak, please fix.  Thanks.
     Chuck_String * a = (Chuck_String *)instantiate_and_initialize_object( SHRED->vm_ref->env()->ckt_string, SHRED );
-    a->set( itoa( i ) );
+    a->set( ck_itoa( i ) );
     RETURN->v_string = a;
 }
 
@@ -784,7 +805,7 @@ CK_DLL_SFUN( ftoa_impl )
     t_CKFLOAT f = GET_NEXT_FLOAT(ARGS);
     t_CKINT p = GET_NEXT_INT(ARGS);
     Chuck_String * a = (Chuck_String *)instantiate_and_initialize_object( SHRED->vm_ref->env()->ckt_string, SHRED );
-    a->set( ftoa( f, (t_CKUINT)p ) );
+    a->set( ck_ftoa( f, (t_CKUINT)p ) );
     RETURN->v_string = a;
 }
 
@@ -796,16 +817,57 @@ CK_DLL_SFUN( ftoi_impl )
 }
 
 // getenv
-// static Chuck_String g_str; // PROBLEM: not thread friendly
 CK_DLL_SFUN( getenv_impl )
 {
-    const char * v = GET_CK_STRING(ARGS)->str().c_str();
+    // get chuck string key
+    Chuck_String * ckstr_key = GET_NEXT_STRING(ARGS);
+    // check for null
+    if( !ckstr_key )
+    {
+        // instantiate
+        Chuck_String * empty = (Chuck_String *)instantiate_and_initialize_object( SHRED->vm_ref->env()->ckt_string, SHRED );
+        // return value
+        RETURN->v_string = empty;
+        // return out of this function
+        return;
+    }
+
+    const char * v = ckstr_key->str().c_str();
     const char * s = getenv( v );
     Chuck_String * a = (Chuck_String *)instantiate_and_initialize_object( SHRED->vm_ref->env()->ckt_string, SHRED );
     a->set( s ? s : "" );
     RETURN->v_string = a;
 }
 
+// getenv with default value fallback | 1.5.0.8 (@ynohtna) added
+CK_DLL_SFUN( getenv2_impl )
+{
+    // get chuck string key and default
+    Chuck_String * ckstr_key = GET_NEXT_STRING(ARGS);
+    Chuck_String * ckstr_default = GET_NEXT_STRING(ARGS);
+    // check for null
+    if( !ckstr_key )
+    {
+        // return default
+        RETURN->v_string = ckstr_default;
+        // return out of this function
+        return;
+    }
+
+    // get key as c str
+    const char * k = ckstr_key->str().c_str();
+    // get env value
+    const char * v = getenv( k );
+    if( v ) {
+        Chuck_String * a = (Chuck_String *)instantiate_and_initialize_object( SHRED->vm_ref->env()->ckt_string, SHRED );
+        a->set( v ? v : "" );
+        RETURN->v_string = a;
+    } else {
+        Chuck_String * d = ckstr_default;
+        RETURN->v_string = d;
+    }
+}
+
 // setenv
 CK_DLL_SFUN( setenv_impl )
 {
@@ -814,7 +876,6 @@ CK_DLL_SFUN( setenv_impl )
     RETURN->v_int = setenv( v1, v2, 1 );
 }
 
-
 // mtof
 CK_DLL_SFUN( mtof_impl )
 {
@@ -902,6 +963,64 @@ CK_DLL_SFUN( scalef_impl )
     RETURN->v_float = dstmin + (dstmax-dstmin) * ((v-srcmin)/(srcmax-srcmin));
 }
 
+// common internal function for range() | 1.5.1.1 (nshaheed)
+static Chuck_ArrayInt * ck_range( t_CKINT start, t_CKINT stop, t_CKINT step, Chuck_VM_Shred * SHRED )
+{
+    // size
+    t_CKINT size = 0;
+    // calculate size based on comparing start and stop
+    if( start < stop && step > 0 ) size = (stop-start)/step + ((stop-start)%step ? 1 : 0);
+    else if( start > stop && step < 0 ) size = (start-stop)/(-step) + ((start-stop)%(-step) ? 1 : 0);
+
+    // allocate array object
+    Chuck_ArrayInt * range = new Chuck_ArrayInt(FALSE, size);
+    // initialize with trappings of Object
+    initialize_object(range, SHRED->vm_ref->env()->ckt_array, SHRED, SHRED->vm_ref);
+
+    // the value
+    t_CKINT value = start;
+    // populate the array
+    for (t_CKINT i = 0; i < size; i++) {
+        range->set(i, value);
+        value += step;
+    }
+
+    // return the array reference
+    return range;
+}
+
+// basic range function - make array with values from [0,stop)
+CK_DLL_SFUN(range1_impl)
+{
+    t_CKINT start = 0;
+    t_CKINT stop = GET_NEXT_INT(ARGS);
+    t_CKINT step = stop > start ? 1 : -1;
+    RETURN->v_object = ck_range( start, stop, step, SHRED );
+}
+
+// start/stop range function - make array with values from [start,stop)
+CK_DLL_SFUN(range2_impl)
+{
+    t_CKINT start = GET_NEXT_INT(ARGS);
+    t_CKINT stop = GET_NEXT_INT(ARGS);
+    t_CKINT step = stop > start ? 1 : -1;
+    RETURN->v_object = ck_range( start, stop, step, SHRED );
+}
+
+// start/stop/step range function - make array with values from [start,stop) with step stride
+CK_DLL_SFUN(range3_impl)
+{
+    t_CKINT start = GET_NEXT_INT(ARGS);
+    t_CKINT stop = GET_NEXT_INT(ARGS);
+    t_CKINT step = GET_NEXT_INT(ARGS);
+
+    // check and flip signs to match with stop-start
+    if( (start < stop && step < 0) ||
+        (start > stop && step > 0 ) ) step = -step;
+
+    RETURN->v_object = ck_range( start, stop, step, SHRED );
+}
+
 
 
 
@@ -1315,7 +1434,10 @@ void LineEvent::prompt( const string & what )
 {
     // set what
     g_le_what = what;
-    if( g_le_what != "" ) g_le_what += " ";
+
+    // remove the extra space
+    // if( g_le_what != "" ) g_le_what += " ";
+
     // signal
     g_le_wait = FALSE;
 }
