@@ -34,12 +34,14 @@
 #include "chuck.h"
 #include "chuck_audio.h"
 #include "chuck_console.h"
+#include "chuck_otf.h"
 #include "util_platforms.h"
 #include "util_string.h"
 #include <signal.h>
 
 #if defined(__PLATFORM_WINDOWS__)
   #include <windows.h>
+  #include <processthreadsapi.h> // GetCurrentProcessId()
 #else
   #include <unistd.h>
   #include <pthread.h>
@@ -56,14 +58,15 @@ using namespace std;
 t_CKBOOL init_shell( Chuck_Shell * shell, Chuck_Shell_UI * ui, Chuck_VM * vm );
 void * shell_cb( void * p );
 t_CKBOOL go( int argc, const char ** argv );
+void EXIT_with_global_cleanup( int exitCode );
 t_CKBOOL global_cleanup();
 void all_stop();
 void all_detach();
 void usage();
 void uh();
 t_CKBOOL get_count( const char * arg, t_CKUINT * out );
-void cb( SAMPLE * in, SAMPLE * out, t_CKUINT numFrames,
-         t_CKUINT numInChans, t_CKUINT numOutChans, void * data );
+static void audio_cb( SAMPLE * in, SAMPLE * out, t_CKUINT numFrames,
+                      t_CKUINT numInChans, t_CKUINT numOutChans, void * data );
 
 // C functions
 extern "C" void signal_int( int sig_num );
@@ -148,14 +151,15 @@ int chuck_main( int argc, const char ** argv )
 
 
 //-----------------------------------------------------------------------------
-// name: cb()
-// desc: audio callback
+// name: audio_cb()
+// desc: audio callback; on audio thread, called through RtAudio;
+//       in turn, this function calls up to chuck to compute next chunk
 //-----------------------------------------------------------------------------
-void cb( SAMPLE * in, SAMPLE * out, t_CKUINT numFrames,
-         t_CKUINT numInChans, t_CKUINT numOutChans, void * data )
+static void audio_cb( SAMPLE * in, SAMPLE * out, t_CKUINT numFrames,
+                      t_CKUINT numInChans, t_CKUINT numOutChans, void * data )
 {
     // TODO: check channel numbers
-    
+
     // call up to ChucK
     the_chuck->run( in, out, numFrames );
 }
@@ -240,6 +244,7 @@ static void version()
 
 
 
+
 //-----------------------------------------------------------------------------
 // name: usage()
 // desc: ...
@@ -250,15 +255,15 @@ void usage()
     CK_FPRINTF_STDERR( "%s", TC::reset().c_str() );
     CK_FPRINTF_STDERR( "\nusage: %s --[%s|%s] [%s] file1 file2 ...\n\n", "chuck", TC::orange("options").c_str(), TC::blue("commands").c_str(), TC::blue("+-=^").c_str() );
     CK_FPRINTF_STDERR( "%s", TC::set_orange().c_str() );
-    CK_FPRINTF_STDERR( "    [options] = halt|loop|audio|silent|dump|nodump|about|probe|\n" );
-    CK_FPRINTF_STDERR( "                channels:<N>|out:<N>|in:<N>|dac:<N>|adc:<N>|driver:<name>|\n" );
-    CK_FPRINTF_STDERR( "                srate:<N>|bufsize:<N>|bufnum:<N>|shell|empty|\n" );
-    CK_FPRINTF_STDERR( "                remote:<hostname>|port:<N>|verbose:<N>|level:<N>|\n" );
-    CK_FPRINTF_STDERR( "                callback|deprecate:{stop|warn|ignore}|chugin-probe|\n" );
+    CK_FPRINTF_STDERR( "    [options] = halt|loop|audio|silent|dump|nodump|about|probe\n" );
+    CK_FPRINTF_STDERR( "                channels:<N>|out:<N>|in:<N>|dac:<N>|adc:<N>|driver:<name>\n" );
+    CK_FPRINTF_STDERR( "                srate:<N>|bufsize:<N>|bufnum:<N>|shell|empty\n" );
+    CK_FPRINTF_STDERR( "                remote:<hostname>|port:<N>|verbose:<N>|level:<N>\n" );
+    CK_FPRINTF_STDERR( "                callback|deprecate:{stop|warn|ignore}|chugin-probe\n" );
     CK_FPRINTF_STDERR( "                chugin-load:{on|off}|chugin-path:<path>|chugin:<name>\n" );
-    CK_FPRINTF_STDERR( "                --color:{on|off}|--no-color\n" );
+    CK_FPRINTF_STDERR( "                color:{on|off}|pid-file:<path>|cmd-listener:{on|off}\n" );
     CK_FPRINTF_STDERR( "%s", TC::set_blue().c_str() );
-    CK_FPRINTF_STDERR( "   [commands] = add|remove|replace|remove.all|status|time|\n" );
+    CK_FPRINTF_STDERR( "   [commands] = add|replace|remove|remove.all|status|time|\n" );
     CK_FPRINTF_STDERR( "                clear.vm|reset.id|abort.shred|exit\n" );
     CK_FPRINTF_STDERR( "       [+-=^] = shortcuts for add, remove, replace, status\n" );
     version();
@@ -267,6 +272,49 @@ void usage()
 
 
 
+// non-NULL when a pid file has been written
+const char * g_ck_pidfile = NULL;
+// current process id
+unsigned long g_ck_pid = 0;
+//-----------------------------------------------------------------------------
+// name: write_pid(filename) | 1.5.1.0 (@ynohtna)
+// desc: writes the host's process id to the given file path
+//       returns true if successful, false otherwise.
+//-----------------------------------------------------------------------------
+bool write_pid( const char * filename )
+{
+#if defined(__PLATFORM_WIN32__)
+    // get process id and store in global variable
+    g_ck_pid = (unsigned long)GetCurrentProcessId();
+#else
+    // get process id and store in global variable
+    g_ck_pid = (unsigned long)getpid();
+#endif
+
+    // open file to write to
+    FILE * pidfile = fopen( filename, "w" );
+    // check file
+    if( !pidfile )
+    {
+        // problem
+        CK_FPRINTF_STDERR( "[chuck]: failed to store process ID %lu in %s\n", g_ck_pid, filename );
+        // clear id
+        g_ck_pid = 0;
+        // report
+        return false;
+    }
+
+    // write to file
+    fprintf( pidfile, "%lu", g_ck_pid );
+    // close the file
+    fclose( pidfile );
+    // done
+    return true;
+}
+
+
+
+
 //-----------------------------------------------------------------------------
 // name: get_count()
 // desc: get the thing after -- or - as number
@@ -330,10 +378,8 @@ extern "C" void signal_pipe( int sig_num )
     {
         // log
         CK_FPRINTF_STDERR( "[chuck]: cleaning up...\n" );
-        // clean up everything!
-        global_cleanup();
-        // later
-        exit( 2 );
+        // done
+        EXIT_with_global_cleanup( 141 ); // 141 -> SIGPIPE
     }
 }
 
@@ -342,7 +388,7 @@ extern "C" void signal_pipe( int sig_num )
 
 //-----------------------------------------------------------------------------
 // name: global_cleanup()
-// desc: ...
+// desc: try to leave the campground better than you found it
 //-----------------------------------------------------------------------------
 t_CKBOOL global_cleanup()
 {
@@ -399,12 +445,40 @@ t_CKBOOL global_cleanup()
     // 1.5.0.0 (ge) | commented
     // s_mutex.release();
 
+    // delete process id file
+    if( g_ck_pidfile )
+    {
+        // log
+        EM_log( CK_LOG_INFO, "cleaning up pid-file '%s'...", g_ck_pidfile );
+        // remove directory entry
+        unlink( g_ck_pidfile );
+        // reset
+        g_ck_pidfile = NULL;
+    }
+    // reset
+    g_ck_pid = 0;
+
     return TRUE;
 }
 
 
 
 
+//-----------------------------------------------------------------------------
+// name: EXIT_with_global_cleanup()
+// desc: exit program after invoking global_cleanup()
+//-----------------------------------------------------------------------------
+void EXIT_with_global_cleanup( int exitCode )
+{
+    // clean up after ourselves, the best we can
+    global_cleanup();
+    // exit program
+    exit( exitCode );
+}
+
+
+
+
 //-----------------------------------------------------------------------------
 // name: all_stop()
 // desc: ...
@@ -413,7 +487,9 @@ void all_stop()
 {
     if( g_enable_realtime_audio )
     {
-        ChuckAudio::shutdown();
+        // argument is time to wait in ms | 1.5.1.3 (ge)
+        // this also seems to get rid of audio stutter on some macOS audio devices
+        ChuckAudio::shutdown( 100 );
     }
     // REFACTOR-2017: TODO: other things? le_cb?
 }
@@ -481,8 +557,8 @@ void * shell_cb( void * p )
     
     // assuming this is absolutely necessary, an assert may be better
     assert( p != NULL );
-    
-    shell = (Chuck_Shell *) p;
+    // get shell object pointer
+    shell = (Chuck_Shell *)p;
     
     //atexit( wait_for_shell );
     
@@ -492,6 +568,7 @@ void * shell_cb( void * p )
     // flag to effect global shutdown from main thread
     g_shutdown_shell = TRUE;
 
+    // log
     EM_log( CK_LOG_INFO, "exiting thread routine for shell..." );
     
     return NULL;
@@ -536,8 +613,8 @@ t_CKBOOL go( int argc, const char ** argv )
     t_CKINT  log_level = CK_LOG_CORE;
     t_CKINT  deprecate_level = 1; // 1 == warn
     t_CKINT  chugin_load = 1; // 1 == auto (variable added 1.3.0.0)
-    // whether to make this new VM the one that receives OTF commands
-    t_CKBOOL update_otf_vm = TRUE;
+    // whether to enable this VM to receive OTF commands over network
+    t_CKBOOL enableOTF = TRUE;
     // whether to print code quotes for compiler message | 1.5.0.5 (ge) added
     t_CKBOOL suppress_error_quote = FALSE;
     // color terminal output | 1.5.0.5 (ge) added
@@ -843,22 +920,46 @@ t_CKBOOL go( int argc, const char ** argv )
                 colorTerminal = TRUE;
             else if( tolower(argv[i]) == "--no-color" )
                 colorTerminal = FALSE;
-            // (added 1.3.0.0)
-            else if( !strcmp( argv[i], "--no-otf" ) )
+            else if( !strncmp(argv[i], "--pid-file:", sizeof("--pid-file:")-1) )
             {
-                // don't use this new vm for otf commands (use the previous one)
-                update_otf_vm = FALSE;
+                // get argument for writing PID file | 1.5.1.0 (@ynohtna) added
+                const char * filename = argv[i] + sizeof("--pid-file:")-1;
+                // write the process ID file |
+                if( write_pid(filename) ) {
+                    // save file name
+                    g_ck_pidfile = filename;
+                }
             }
             else if( !strcmp( argv[i], "--probe" ) )
                 probe = TRUE;
             else if( !strcmp( argv[i], "--chugin-probe" ) )
                 probe_chugs = TRUE;
+            else if( !strcmp( argv[i], "--disable-error-show-code" ) )
+                suppress_error_quote = TRUE;
+            else if( !strncmp(argv[i], "--cmd-listener:", sizeof("--cmd-listener:")-1) )
+            {
+                // get the rest
+                string arg = tolower(argv[i]+sizeof("--cmd-listener:")-1);
+                if( arg == "off" ) enableOTF = false;
+                else if( arg == "on" || arg == "auto") enableOTF = true;
+                else
+                {
+                    // error
+                    errorMessage1 = "invalid arguments for '--cmd-listener:'...";
+                    errorMessage2 = "...(looking for ON, OFF, or AUTO)";
+                    break;
+                }
+            }
+            // (added 1.3.0.0)
+            else if( !strcmp( argv[i], "--no-cmd-listener" ) )
+            {
+                // don't use this new vm for otf commands (use the previous one)
+                enableOTF = FALSE;
+            }
             else if( !strcmp( argv[i], "--poop" ) )
                 uh();
             else if( !strcmp( argv[i], "--caution-to-the-wind" ) )
                 g_enable_system_cmd = TRUE;
-            else if( !strcmp( argv[i], "--disable-error-show-code" ) )
-                suppress_error_quote = TRUE;
             else if( !strcmp(argv[i], "--help") || !strcmp(argv[i], "-h")
                     || !strcmp(argv[i], "--about") )
             {
@@ -879,10 +980,17 @@ t_CKBOOL go( int argc, const char ** argv )
                 
                 // do it
                 if( otf_send_cmd( argc, argv, i, g_otf_dest, g_otf_port, &is_otf ) )
-                    exit( 0 );
-                
+                {
+                    // done
+                    EXIT_with_global_cleanup( 0 );
+                }
+
                 // is otf
-                if( is_otf ) exit( 1 );
+                if( is_otf )
+                {
+                    // done
+                    EXIT_with_global_cleanup( 1 );
+                }
                 
                 // done
                 errorMessage1 = string("invalid flag '") + argv[i] + "' (see --help/-h)";
@@ -911,6 +1019,10 @@ t_CKBOOL go( int argc, const char ** argv )
 
     // set log level so things can start logging
     the_chuck->setLogLevel( log_level );
+
+    // log PID file output
+    if( g_ck_pidfile )
+    { EM_log( CK_LOG_INFO, "process ID '%lu' stored in '%s'...", g_ck_pid, g_ck_pidfile ); }
     //-----------------------------------------------------------------
 
     // check for error message from command-line arguments
@@ -921,8 +1033,8 @@ t_CKBOOL go( int argc, const char ** argv )
     {
         // see if need to print usage
         if( doAbout ) usage();
-        // bail for now
-        exit(1);
+        // done
+        EXIT_with_global_cleanup( 1 );
     }
 
     //-----------------------------------------------------------------
@@ -930,16 +1042,20 @@ t_CKBOOL go( int argc, const char ** argv )
     //-----------------------------------------------------------------
     if( doAbout )
     {
+        // print usage info
         usage();
-        exit( 2 );
+        // done
+        EXIT_with_global_cleanup( 0 );
     }
     //-----------------------------------------------------------------
     // version
     //-----------------------------------------------------------------
     if( doVersion )
     {
+        // print version
         version();
-        exit( 2 );
+        // done
+        EXIT_with_global_cleanup( 0 );
     }
     //-----------------------------------------------------------------
     // probe
@@ -965,8 +1081,10 @@ t_CKBOOL go( int argc, const char ** argv )
         // probe HID devices
         HidInManager::probeHidIn();
 
-        // done probe
-        exit( 0 );
+        // restore log level
+        the_chuck->setLogLevel( log_level );
+        // done
+        EXIT_with_global_cleanup( 0 );
     }
 
     // set caution to wind
@@ -986,10 +1104,13 @@ t_CKBOOL go( int argc, const char ** argv )
     // set adaptive size
     if( adaptive_size < 0 ) adaptive_size = buffer_size;
 
+    // no files or --loop
     if( !files && vm_halt && !g_enable_shell && !probe_chugs )
     {
+        // print error
         EM_error2( 0, "no input files... (try --help/-h)" );
-        exit( 1 );
+        // done
+        EXIT_with_global_cleanup( 1 );
     }
 
     // shell initialization without vm
@@ -999,20 +1120,25 @@ t_CKBOOL go( int argc, const char ** argv )
         g_shell = new Chuck_Shell;
         // initialize
         if( !init_shell( g_shell, new Chuck_Console(), NULL ) )
-            exit( 1 );
+        {
+            // done
+            EXIT_with_global_cleanup( 1 );
+        }
         // no vm is needed, just start running the shell now
         g_shell->run();
         // clean up
         CK_SAFE_DELETE( g_shell );
         // done
-        exit( 0 );
+        EXIT_with_global_cleanup( 0 );
     }
     
     // make sure vm
     if( no_vm )
     {
+        // error
         CK_FPRINTF_STDERR( "[chuck]: '--empty' can only be used with shell...\n" );
-        exit( 1 );
+        // done
+        EXIT_with_global_cleanup( 1 );
     }
     
     // find dac_name if appropriate (added 1.3.0.0)
@@ -1028,7 +1154,8 @@ t_CKBOOL go( int argc, const char ** argv )
         {
             CK_FPRINTF_STDERR( "[chuck]: unable to find dac '%s'...\n", dac_name.c_str() );
             CK_FPRINTF_STDERR( "[chuck]: | (try --probe to enumerate audio device info)\n" );
-            exit( 1 );
+            // done
+            EXIT_with_global_cleanup( 1 );
         }
     }
     
@@ -1045,7 +1172,8 @@ t_CKBOOL go( int argc, const char ** argv )
         {
             CK_FPRINTF_STDERR( "[chuck]: unable to find adc '%s'...\n", adc_name.c_str() );
             CK_FPRINTF_STDERR( "[chuck]: | (try --probe to enumerate audio device info)\n" );
-            exit( 1 );
+            // done
+            EXIT_with_global_cleanup( 1 );
         }
     }
 
@@ -1068,9 +1196,8 @@ t_CKBOOL go( int argc, const char ** argv )
         // probe chugins (.chug and .ck modules)
         // print/log what ChucK would load with current settings
         the_chuck->probeChugins();
-
-        // done with probe
-        exit( 0 );
+        // done
+        EXIT_with_global_cleanup( 0 );
     }
 
     // log
@@ -1078,7 +1205,6 @@ t_CKBOOL go( int argc, const char ** argv )
     // push
     EM_pushlog();
 
-
     //--------------------------- AUDIO I/O SETUP -----------------------------
     // log
     EM_log( CK_LOG_SYSTEM, "initializing system settings..." );
@@ -1095,7 +1221,7 @@ t_CKBOOL go( int argc, const char ** argv )
     {
         // initialize real-time audio
         t_CKBOOL retval = ChuckAudio::initialize( dac, adc, dac_chans, adc_chans,
-            srate, buffer_size, num_buffers, cb, (void *)the_chuck, force_srate, audio_driver.c_str() );
+            srate, buffer_size, num_buffers, audio_cb, (void *)the_chuck, force_srate, audio_driver.c_str() );
         // check return code
         if( !retval )
         {
@@ -1106,8 +1232,9 @@ t_CKBOOL go( int argc, const char ** argv )
             EM_log( CK_LOG_SYSTEM, "| (use --help for more details)" );
             // pop
             EM_poplog();
+
             // done
-            exit( 1 );
+            EXIT_with_global_cleanup( 1 );
         }
 
         // these could have been updated during initialization
@@ -1138,7 +1265,9 @@ t_CKBOOL go( int argc, const char ** argv )
     the_chuck->setParam( CHUCK_PARAM_VM_ADAPTIVE, adaptive_size );
     the_chuck->setParam( CHUCK_PARAM_VM_HALT, (t_CKINT)(vm_halt) );
     the_chuck->setParam( CHUCK_PARAM_OTF_PORT, g_otf_port );
-    the_chuck->setParam( CHUCK_PARAM_OTF_ENABLE, (t_CKINT)TRUE );
+    the_chuck->setParam( CHUCK_PARAM_OTF_ENABLE, (t_CKINT)enableOTF );
+    // only print warning if say --loop is specified, signalling interest in using this VM as a server | 1.5.1.2
+    the_chuck->setParam( CHUCK_PARAM_OTF_PRINT_WARNINGS, (t_CKINT)(vm_halt == FALSE) );
     the_chuck->setParam( CHUCK_PARAM_DUMP_INSTRUCTIONS, (t_CKINT)dump );
     the_chuck->setParam( CHUCK_PARAM_AUTO_DEPEND, (t_CKINT)auto_depend );
     the_chuck->setParam( CHUCK_PARAM_DEPRECATE_LEVEL, deprecate_level );
@@ -1150,8 +1279,10 @@ t_CKBOOL go( int argc, const char ** argv )
     // initialize
     if( !the_chuck->init() )
     {
+        // print error
         CK_FPRINTF_STDERR( "[chuck]: failed to initialize...\n" );
-        exit( 1 );
+        // done
+        EXIT_with_global_cleanup( 1 );
     }
 
     // timestamp
@@ -1205,7 +1336,8 @@ t_CKBOOL go( int argc, const char ** argv )
         {
             // error
             CK_FPRINTF_STDERR( "[chuck]: error setting thread priority...\n" );
-            exit( 1 );
+            // done
+            EXIT_with_global_cleanup( 1 );
         }
     }
 
@@ -1217,7 +1349,10 @@ t_CKBOOL go( int argc, const char ** argv )
         g_shell = new Chuck_Shell;
         // initialize
         if( !init_shell( g_shell, new Chuck_Console(), the_chuck->vm() ) )
-            exit( 1 );
+        {
+            // done
+            EXIT_with_global_cleanup( 1 );
+        }
 
         // flag
         g_shutdown_shell = FALSE;
